# coding=utf-8
from osv import fields,osv,orm

class crm_case(osv.osv):
	_inherit = "crm.case"

	def _canal_id_list(self, cr, uid, context={}):
		cr.execute('SELECT id, name FROM res_partner_canal')
		return cr.fetchall()

	def _canal_id_default(self, cr, uid, context={}):
		cr.execute("SELECT id FROM res_partner_canal WHERE name='E-mail'")
		if cr.rowcount > 0:
			return cr.fetchone()[0]
		else:
			return False

	def _status_list(self, cr, uid, context={}):
		cr.execute('SELECT key, name FROM crm_status')
		return cr.fetchall()
	
	_columns = {
		'next_step': fields.char('Title', size=128),
		'date': fields.datetime('Start Date'),
		'canal_id': fields.selection(_canal_id_list, 'Channel',size=64),

		'user_ids': fields.many2many('res.users', 'crm_user_rel', 'user_id', 'case_id', 'Usuários Envolvidos', select=True),
		'partner_ids': fields.many2many('res.partner', 'crm_partner_rel', 'partner_id', 'case_id', 'Parceiros Relacionados', select=True),

		'status_id': fields.selection(_status_list, 'Status',size=64, select=True),
	}

	_defaults = {
		'priority': lambda *a: '3',
		'canal_id': _canal_id_default,		
	}

	# Escreve a description dentro do Case Log, ao invés de criar um registro de History			
	def case_log(self, cr, uid, ids,context={}, email=False, *args):
		cases = self.browse(cr, uid, ids)
		self.__history(cr, uid, cases, 'Historize', history=True, email=email)
		return self.write(cr, uid, ids, {'description': False, 'som': False, 'next_step':False})
			
	def __history(self, cr, uid, cases, keyword, history=False, email=False, context={}):
		for case in cases:
			if case.next_step != False:
				name = case.next_step
			else:
				name = keyword
				
			data = {
				'name': name,
				'som': case.som.id,
				'canal_id': case.canal_id,
				'user_id': uid,
				'case_id': case.id,
				'description': case.description,
			}
			obj = self.pool.get('crm.case.log')
			obj.create(cr, uid, data, context)
		return True
	
	def __log(self, cr, uid, cases, keyword, context={}):
		if not self.pool.get('res.partner.event.type').check(cr, uid, 'crm_case_'+keyword):
			return False
		for case in cases:
			if case.partner_id:
				name = 'Case ' + keyword + ': ' + case.name
				if len(name) > 64:
					name = name[:61] + '...'
				self.pool.get('res.partner.event').create(cr, uid, {
					'name': name,
					'som':(case.som or False) and case.som.id,
					'description':case.description,
					'partner_id':case.partner_id.id,
					'date':case.date,
					'canal_id': (case.canal_id or False),
					'user_id':uid,
				})
		return True

	# Alterado pois retiramos o campo email_from
	def onchange_partner_id(self, cr, uid, ids, part):
		if not part:
			return {'value':{'partner_address_id': False}}
		addr = self.pool.get('res.partner').address_get(cr, uid, [part], ['contact'])
		data = {'partner_address_id':addr['contact']}
		return {'value':data}

	# Alterado pois retiramos o campo email_from
	def onchange_partner_address_id(self, cr, uid, ids, part):
		if not part:
			return {'value':{}}
		data = {}
		return {'value':data}
crm_case()

class crm_case_log(osv.osv):
	_inherit = "crm.case.log"

	def _canal_id_list(self, cr, uid, context={}):
		cr.execute('SELECT id, name FROM res_partner_canal')
		return cr.fetchall()

	_columns = {
		'user_id': fields.many2one('res.users', 'User', readonly=True),
		'name': fields.char('Title', size=128),
		'description': fields.text('Description'),
		'canal_id': fields.selection(_canal_id_list, 'Channel',size=64),
	}
crm_case_log()

class crm_status(osv.osv):
	_name = "crm.status"
	_columns = {
		'name': fields.char('Name', size=64, select=True, required=True ),
		'key': fields.char('Key', size=64, select=True, required=True ),
	}
crm_status()
