# coding=utf-8
from osv import osv, fields

import pooler
import time
from mx import DateTime
import netsvc

# Inherited Class from mrp.py:140
class mrp_bom(osv.osv):
	_inherit  = 'mrp.bom'
	
	# Criar o campo 'bom_ids' para armazenar os Parents BoM (cria a tabela mrp_bom_rel por padrão) em uma
	# relação many2many. Altera o bom_lines também para many2many, para tornar compatível.
	_columns = {
		'bom_lines': fields.many2many('mrp.bom', 'mrp_bom_rel', 'parent_id', 'child_id', 'Childs BoM'),
		'bom_ids': fields.many2many('mrp.bom', 'mrp_bom_rel', 'child_id', 'parent_id', 'Parents BoM'),
	}
	
	# Originalmente e função _bom_explode ignora os BoM que tem filhos, e só considera os próprios filhos.
	# Redefinimos para que todo o primeiro nível de BoMs seja considerado (independente se tem filhos ou não)
	# e ignoramos completamente os filhos. 

	# Original Function from mrp.py:223
	def _bom_explode (self, cr, uid, bom, factor, properties, addthis=False, level=10):
		factor = factor / (bom.product_efficiency or 1.0)

		# Necessário para a função _bom_explode, original de mrp.py:302
		def rounding(f, r):
			if not r:
				return f
			return round(f / r) * r

		factor = rounding(factor, bom.product_rounding)
		if factor<bom.product_rounding:
			factor = bom.product_rounding
		result = [] #é usado pela função action_compute em mrp.production.product.line
		result2 = [] #é usado pela função action_compute em mrp.production.workcenter.line

		for bom2 in bom.bom_lines:
			result.append(
			{
				'name': bom2.product_id.name,
				'product_id': bom2.product_id.id,
				'product_qty': bom2.product_qty * factor,
				'product_uom': bom2.product_uom.id,
			})

			if bom2.routing_id:
				for wc_use in bom2.routing_id.workcenter_lines:
					wc = wc_use.workcenter_id
					d, m = divmod(factor, wc_use.workcenter_id.capacity_per_cycle)
					cycle = (d + (m and 1.0 or 0.0)) * wc_use.cycle_nbr
					result2.append({
						'name': bom2.routing_id.name,
						'workcenter_id': wc.id,
						'sequence': level,
						'cycle': cycle,
						'hour': wc_use.hour_nbr + (wc.time_start+wc.time_stop+cycle*wc.time_cycle) * (wc.time_efficiency or 1.0),
					})

		return result, result2
		
	# Na nossa definição uma BoM com pai (bom_id != null) PODE ser usado para produção.
	# Original Function from mrp.py:207	
	def _bom_find (self, cr, uid, product_id, product_uom, properties = []):
		bom_result = False
		cr.execute('SELECT id FROM mrp_bom WHERE product_id=%d ORDER BY sequence ASC', (product_id,))
		ids = map(lambda x: x[0], cr.fetchall())
		max_prop = 0
		result = False
		for bom in self.pool.get('mrp.bom').browse(cr, uid, ids):
			prop = 0
			for prop_id in bom.property_ids:
				if prop_id.id in properties:
					prop+=1
			if (prop>max_prop) or ((max_prop==0) and not result):
				result = bom.id
		return result
		
	# Verifica se a BoM tem o próprio produto como filho ou pai.
	def check_recursion(self, cr, uid, vals, product_id):
		if vals.has_key('bom_lines'):
			bom_ids = vals['bom_lines'][0][2]
			for b in self.browse(cr, uid, bom_ids):
				if b.product_id.id == product_id:
					raise osv.except_osv('Error','A BoM não pode conter o próprio produto como filho!')
		if vals.has_key('bom_ids'):
			bom_ids = vals['bom_ids'][0][2]
			for b in self.browse(cr, uid, bom_ids):
				if b.product_id.id == product_id:
					raise osv.except_osv('Error','A BoM não pode conter o próprio produto como pai!')
	def write(self, cr, uid, ids, vals, context=None):
		for bom in self.browse(cr, uid, ids):
			self.check_recursion(cr, uid, vals, bom.product_id.id)
		res = osv.osv.write(self, cr, uid, ids, vals, context)
		return res
	def create(self, cr, uid, vals, context=None):
		self.check_recursion(cr, uid, vals, vals['product_id'])
		res = osv.osv.create(self, cr, uid, vals, context)		
		return res
				
mrp_bom()

