# coding=utf-8
from osv import osv, fields

import pooler
import time
from mx import DateTime
import netsvc

# Inherited Class from mrp.py:635
class mrp_procurement(osv.osv):
	_inherit = 'mrp.procurement'
	_columns = {
		'origin': fields.char('Origin', size=64, readonly=True),
		'from_sale_order': fields.boolean('From Sale Order', readonly=True, required=True),

		# Aponta para uma produção. Não é obrigatório. Serve para a rastreabilidade do desfazer produção.
		'production_id': fields.many2one('mrp.production', 'Production', readonly=True),
	}
	_defaults = {
		'from_sale_order': lambda *a: False,
	}
	# Seta as novas Locations de Produção nas produções automáticas.
	def action_produce_assign_product(self, cr, uid, ids, context={}):
		produce_id = False
		for procurement in self.browse(cr, uid, ids):
			res_id = procurement.move_id.id
			loc_id = procurement.location_id.id

			if procurement.location_id.warehouse_id.id:
				warehouse_id = procurement.location_id.warehouse_id.id
			else:
				raise UserWarning, 'No warehouse_id in location "%s" (id:%d)' % (procurement.location_id.name, procurement.location_id.id)			
			
			res = self.pool.get('stock.warehouse').read(cr, uid, [warehouse_id], ['lot_production_id','lot_origin_id'])[0]		
			src = res['lot_production_id'][0] # Em Produção
			stock = res['lot_origin_id'][0] # Insumos
			dest = stock # Volta para os Insumos
						
			newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.produce_delay or 0.0)

			produce_id = self.pool.get('mrp.production').create(cr, uid, {
				'origin': procurement.origin,
				'product_id': procurement.product_id.id,
				'product_qty': procurement.product_qty,
				'product_uom': procurement.product_uom.id,
				'warehouse_id': warehouse_id,
				'location_stock_id': stock,
				'location_src_id': src,
				'location_dest_id': dest,
				'date_planned': newdate,
				'move_prod_id': res_id,
				'procurement_id': procurement.id, #Rastreabilidade do desfazer produção
			})
									
			self.write(cr, uid, [procurement.id], {'state':'running'})
			bom_result = self.pool.get('mrp.production').action_compute(cr, uid,
					[produce_id], properties=[x.id for x in procurement.property_ids])
			wf_service = netsvc.LocalService("workflow")
			wf_service.trg_validate(uid, 'mrp.production', produce_id, 'button_confirm', cr)

		return produce_id
		
	# Seta a nova Locations das POs nos Procurement Make to Order
	def action_po_assign(self, cr, uid, ids):
		purchase_id = False
		for procurement in self.browse(cr, uid, ids):
			res_id = procurement.move_id.id
			partner = procurement.product_id.seller_ids[0].name
			partner_id = partner.id
			address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
			pricelist_id = partner.property_product_pricelist_purchase.id

			uom_id = procurement.product_id.uom_po_id.id

			qty = self.pool.get('product.uom')._compute_qty(cr, uid, procurement.product_uom.id, procurement.product_qty, uom_id)
			if procurement.product_id.seller_ids[0].qty:
				qty=max(qty,procurement.product_id.seller_ids[0].qty)

			price = self.pool.get('product.pricelist').price_get(cr, uid, [pricelist_id], procurement.product_id.id, qty, False, {'uom': uom_id})[pricelist_id]

			newdate = DateTime.strptime(procurement.date_planned, '%Y-%m-%d') - DateTime.RelativeDateTime(days=procurement.product_id.product_tmpl_id.seller_delay or 0.0)
			line = {
				'name': procurement.product_id.name,
				'product_qty': qty,
				'product_id': procurement.product_id.id,
				'product_uom': uom_id,
				'price_unit': price,
				'date_planned': newdate.strftime('%Y-%m-%d'),
				'taxes_id': [(6, 0, [x.id for x in procurement.product_id.product_tmpl_id.supplier_taxes_id])],
				'move_dest_id': res_id,
			}

			if procurement.location_id.warehouse_id.id:
				loc_id = procurement.location_id.warehouse_id.id
				warehouse_id = loc_id
			else:
				raise UserWarning, 'No warehouse_id in location "%s" (id:%d)' % (procurement.location_id.name, procurement.location_id.id)
			if warehouse_id:
				res = self.pool.get('stock.warehouse').read(cr, uid, [warehouse_id], ['lot_input_id'])[0]		
				loc_id = res['lot_input_id'][0] # Recebimento
							
			purchase_id = self.pool.get('purchase.order').create(cr, uid, {
				'origin': procurement.origin,
				'partner_id': partner_id,
				'partner_address_id': address_id,
				'location_id': loc_id,
				'pricelist_id': pricelist_id,
				'order_line': [(0,0,line)],
				'procurement_id': procurement.id, #Rastreabilidade do desfazer produção
			})
			self.write(cr, uid, [procurement.id], {'state':'running', 'purchase_id':purchase_id})
		return purchase_id


	# Chama a nova função _procure_confirm logo abaixo. Originalmente _procure_confirm ficava em outro arquivo.
	# Original Function from mrp.py:936	
	def run_procure_confirm(self, cr, uid, user_id=False, schedule_cycle=1.0,\
			po_cycle=1.0, po_lead=1.0, security_lead=50.0, picking_lead=1.0, \
			use_new_cursor=False, context=None):
		
		if not context:
			context={}
		_procure_confirm(self, cr, uid, schedule_cycle=schedule_cycle,\
				po_cycle=po_cycle, po_lead=po_lead, security_lead=security_lead,\
				picking_lead=picking_lead, user_id=user_id, use_new_cursor=use_new_cursor,\
				context=context)
				
	# Originalmente o Order Point criar Moves junto com as Purchase Orders, apesar do POs estarem em Draft.
	# Isso é uma gambiarra da mrp original. Nossa modificação NÃO cria esse Move, e altera o OP para verificar
	# também as POs em Draft, para descobrir se precisa comprar ou não.
	def run_orderpoint_confirm(self, cr, uid, automatic=False, use_new_cursor=False,\
			context=None, user_id=False):
		'''
		use_new_cursor: False or the dbname
		'''
		if not context:
			context={}
		_procure_orderpoint_confirm(self, cr, uid, automatic=automatic,\
				use_new_cursor=use_new_cursor, context=context, user_id=user_id)

mrp_procurement()

# Original de mrp/wizard/schedulers.py:170
def _procure_orderpoint_confirm(self, cr, uid, automatic=False,\
		use_new_cursor=False, context=None, user_id=False):
	if not context:
		context={}
	if use_new_cursor:
		cr = pooler.get_db(use_new_cursor).cursor()
	pool = pooler.get_pool(cr.dbname)
	orderpoint_obj = pool.get('stock.warehouse.orderpoint')
	location_obj = pool.get('stock.location')
	procurement_obj = pool.get('mrp.procurement')
	request_obj = pool.get('res.request')
	wf_service = netsvc.LocalService("workflow")
	report = []
	offset = 0
	ids = [1]
	
	if automatic:
		create_automatic_op(cr, uid, context=context)
	while ids:
		cr.execute('SELECT id ' \
				'FROM stock_warehouse_orderpoint ' \
				'WHERE active OFFSET %d LIMIT 100', (offset,))
		ids = map(lambda x: x[0], cr.fetchall())
		for op in orderpoint_obj.browse(cr, uid, ids):

			try:
				prods = location_obj._product_virtual_get(cr, uid,
						op.warehouse_id.lot_stock_id.id, [op.product_id.id],
						{'uom': op.product_uom.id})[op.product_id.id]
						
				# Computa as quantidade das POs em aberto
				prods = prods + _moves_not_approved(cr,op.product_id.id)
				if prods < op.product_min_qty:
					# order the quantity necessary to get to max(min_qty, max_qty)
					qty = max(op.product_min_qty, op.product_max_qty)-prods
					reste = qty % op.qty_multiple
					if reste>0:
						qty += op.qty_multiple-reste

					newdate = DateTime.now() + DateTime.RelativeDateTime(
							days=op.product_id.seller_delay)

					# Cria POs se for Buy
					if op.product_id.supply_method == 'buy':
						data_ok = True

						partner = op.product_id.seller_ids[0].name
						if not partner:
							data_ok = False
							report.append('%s: Fornecedor não definido!' % op.product_id.name)
						partner_id = partner.id

						address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
						if not address_id:
							data_ok = False
							report.append('%s: Contato (endereço) do fornecedor não definido!' % op.product_id.name)

						pricelist_id = partner.property_product_pricelist_purchase.id
						if not pricelist_id:
							data_ok = False
							report.append('%s: Pricelist não definida!' % op.product_id.name)

						uom_id = op.product_id.uom_po_id.id
						if not uom_id:
							data_ok = False
							report.append('%s: UOM não definido!' % op.product_id.name)					
						uom_factor = op.product_id.uom_po_id.factor
						
						if op.product_id.seller_ids[0].qty:
							qty=max(qty,op.product_id.seller_ids[0].qty)

						if uom_factor<1:
							import math
							qty = math.ceil(qty * uom_factor)

						if qty<=0:
							data_ok = False
							report.append('%s: Quantidade incorreta (%.2f)' % (op.product_id.name,qty))
						
						price = self.pool.get('product.pricelist').price_get(cr, uid, [pricelist_id], op.product_id.id, qty, False, {'uom': uom_id})[pricelist_id]
						if price<0:
							data_ok = False
							report.append('%s: Preço incorreto (%.2f)' % (op.product_id.name,price))
						
						line = {
							'name': op.product_id.name,
							'product_qty': qty,
							'product_id': op.product_id.id,
							'product_uom': uom_id,
							'price_unit': price,
							'date_planned': DateTime.today().strftime('%Y-%m-%d'),
							'taxes_id': [(6, 0, [x.id for x in op.product_id.product_tmpl_id.supplier_taxes_id])],
						}
						
						
						warehouse_id = op.warehouse_id.id
						res = self.pool.get('stock.warehouse').read(cr, uid, [warehouse_id], ['lot_input_id'])[0]		
						loc_id = res['lot_input_id'][0] # Recebimento
						
						if data_ok:
							purchase_id = self.pool.get('purchase.order').create(cr, uid, {
								'origin': op.name,
								'partner_id': partner_id,
								'partner_address_id': address_id,
								'location_id': loc_id,
								'pricelist_id': pricelist_id,
								'order_line': [(0,0,line)]
							})
						else:
							report = ['# Erro no Order Point (Minimum Stock Rules):'] + report
						
					# Cria Prodoctions se for Produce
					elif op.product_id.supply_method == 'produce':

						produce_id = self.pool.get('mrp.production').create(cr, uid, {
							'origin': op.name,
							'product_id': op.product_id.id,
							'product_qty': qty,
							'product_uom': op.product_uom.id,
							'location_src_id': op.warehouse_id.lot_production_id.id,
							'location_stock_id': op.warehouse_id.lot_origin_id.id,
							'location_dest_id': op.warehouse_id.lot_origin_id.id,
							'warehouse_id': op.warehouse_id.id,
							'date_planned': DateTime.today().strftime('%Y-%m-%d'),
							#'move_prod_id': res_id,
						})
						
						bom_result = self.pool.get('mrp.production').action_compute(cr, uid,
								[produce_id], [])
						wf_service = netsvc.LocalService("workflow")
						wf_service.trg_validate(uid, 'mrp.production', produce_id, 'button_confirm', cr)

					else:
						continue
			except Exception, e:
				report.append('OP %d:\n%s\n' % (op.id, str(e)))
		offset += len(ids)
		if use_new_cursor:
			cr.commit()
	if user_id and report:
		request_obj.create(cr, uid, {
			'name': 'Orderpoint report.',
			'act_from': user_id,
			'act_to': user_id,
			'body': '\n'.join(report)
			})
	if use_new_cursor:
		cr.commit()
		cr.close()
	return {}

# Retorna a quantidade em ordens de compra e produção que não geraram moves para um dado produto.	
def _moves_not_approved (cr,product_id):
	qty = 0
	cr.execute("SELECT t1.product_qty,t3.factor FROM purchase_order_line AS t1, purchase_order AS t2, product_uom AS t3 WHERE t2.id=t1.order_id AND t2.state IN ('draft','confirmed') AND t3.id=t1.product_uom AND t1.product_id=%i" % product_id)
	POs = cr.fetchall()
	for po in POs:
		qty += po[0] * (1/po[1])

	cr.execute("SELECT t1.product_qty,t2.factor FROM mrp_production AS t1, product_uom AS t2 WHERE t2.id=t1.product_uom AND t1.state IN ('draft') AND t1.product_id=%i" % product_id)
	MOs = cr.fetchall()
	for mo in MOs:
		qty += mo[0] * (1/mo[1])

	return float('%.2f' % qty)

# Novo relatório de Exception Procurements com relação mais detalhada dos produtos que faltam.
# Original function from mrp/wizard/schedulers.py:34
def _procure_confirm(self, cr, uid, schedule_cycle=1.0, po_cycle=1.0,\
		po_lead=1.0, security_lead=50.0, picking_lead=1.0, user_id=False,\
		use_new_cursor=False, context=None):
	'''
	use_new_cursor: False or the dbname
	'''
	if not context:
		context={}

	if use_new_cursor:
		cr = pooler.get_db(use_new_cursor).cursor()
	wf_service = netsvc.LocalService("workflow")

	cr.execute('select id from mrp_procurement where state=%s order by date_planned', ('exception',))
	ids = map(lambda x:x[0], cr.fetchall())
	for id in ids:
		wf_service.trg_validate(uid, 'mrp.procurement', id, 'button_restart', cr)
	if use_new_cursor:
		cr.commit()

	po_time = po_cycle + po_lead
	maxdate = DateTime.now() + DateTime.RelativeDateTime(days=schedule_cycle + security_lead)
	start_date = time.strftime('%Y-%m-%d, %Hh %Mm %Ss')
	offset = 0

	# New exception report list
	exception = {}
	exception_total = 0

	report = []
	report_total = 0
	report_except = 0
	report_later = 0
	ids = [1]
	while len(ids):
		cr.execute("SELECT id FROM mrp_procurement WHERE state=%s and procure_method=%s AND from_sale_order<>'true' ORDER BY date_planned LIMIT 500 OFFSET %d", ('confirmed','make_to_order',offset))
		ids = map(lambda x:x[0], cr.fetchall())
		for proc in pooler.get_pool(cr.dbname).get('mrp.procurement').browse(cr, uid, ids):
			if proc.product_id.supply_method=='produce':
				wf_service.trg_validate(uid, 'mrp.procurement', proc.id, 'button_check', cr)
			else:
				if (maxdate + DateTime.RelativeDateTime(days=(proc.product_id.seller_delay or 0 + po_time))).strftime('%Y-%m-%d')>=proc.date_planned:
					wf_service.trg_validate(uid, 'mrp.procurement', proc.id, 'button_check', cr)
				else:
					offset+=1
					report_later += 1
			cr.execute('select name,state from mrp_procurement where id=%d', (proc.id,))
			name,state = cr.fetchone()
			if state=='exception':
				proc_execp = 'PROC %d: on order - %3.2f %-5s - %s' % (proc.id,proc.product_qty,proc.product_uom.name, proc.product_id.name)
				report.append(proc_execp)
				report_except +=1
				pid = proc.product_id.id
				if exception.has_key(pid):
					exception[pid]['qty'] += proc.product_qty
					exception[pid]['proc'].append(proc_execp)
				else:

					virtual = self.pool.get('product.product').read(cr, uid, [pid], ['virtual_available','code'])
					exception[pid] =  {'name': proc.product_id.name, 'uom': proc.product_uom.name, 'qty': proc.product_qty, 'virtual_available': virtual[0]['virtual_available'], 'code': virtual[0]['code'], 'proc': [proc_execp] }

			report_total +=1
			if use_new_cursor:
				cr.commit()

	offset = 0
	ids = [1]
	while len(ids):
		ids = pooler.get_pool(cr.dbname).get('mrp.procurement').search(cr, uid, [('state','=','confirmed'),('procure_method','=','make_to_stock'),('from_sale_order','<>','true')], offset=offset)
		for proc in pooler.get_pool(cr.dbname).get('mrp.procurement').browse(cr, uid, ids):
			if (maxdate + DateTime.RelativeDateTime(days=picking_lead)).strftime('%Y-%m-%d') >= proc.date_planned:
				wf_service.trg_validate(uid, 'mrp.procurement', proc.id, 'button_check', cr)
				cr.execute('select name,state from mrp_procurement where id=%d', (proc.id,))
				name,state = cr.fetchone()
				if state=='exception':
					proc_execp = 'PROC %d: from stock - %3.2f %-5s - %s' % (proc.id,proc.product_qty,proc.product_uom.name, proc.product_id.name,)
					report.append(proc_execp)
					report_except +=1
					pid = proc.product_id.id
					if exception.has_key(pid):
						exception[pid]['qty'] += proc.product_qty
						exception[pid]['proc'].append(proc_execp)
					else:

						virtual = self.pool.get('product.product').read(cr, uid, [pid], ['virtual_available','code'])
						exception[pid] =  {'name': proc.product_id.name, 'uom': proc.product_uom.name, 'qty': proc.product_qty, 'virtual_available': virtual[0]['virtual_available'], 'code': virtual[0]['code'], 'proc': [proc_execp] }
			else:
				report_later +=1
			report_total +=1
			if use_new_cursor:
				cr.commit()
		offset += len(ids)
	end_date = time.strftime('%Y-%m-%d, %Hh %Mm %Ss')
	if user_id:
		request = pooler.get_pool(cr.dbname).get('res.request')
		summary = '''# Here is the procurement scheduling report.

Computation Started; %s
Computation Finnished; %s

Total procurement: %d
Exception procurement: %d
Not run now procurement: %d

# Exceptions;
'''% (start_date,end_date,report_total, report_except,report_later)
		summary += '\n'.join(report)


		
		falker_report = []
		falker_report2 = []
		for e in exception:
			if exception[e]['virtual_available'] < 0:
				execp = '[%s]; %s; %s; %.2f' % (exception[e]['code'], exception[e]['name'], exception[e]['uom'], -exception[e]['virtual_available'])
				for proc in exception[e]['proc']:
					execp += '\n      %s' % (proc)
				execp += '\n      Necessário: %.2f   -   Disponível: %.2f   =   Falta: %.2f\n' % (exception[e]['qty'], exception[e]['qty'] + exception[e]['virtual_available'], -exception[e]['virtual_available'])
				falker_report.append(execp)
			else:
				execp = '*[%s]; %s; %s; %.2f' % (exception[e]['code'], exception[e]['name'], exception[e]['uom'], exception[e]['virtual_available'])
				for proc in exception[e]['proc']:
					execp += '\n      %s' % (proc)
				execp += '\n      Disponível: %.2f   -   Necessário: %.2f   =   Saldo: %.2f\n' % (exception[e]['qty'] + exception[e]['virtual_available'], exception[e]['qty'], exception[e]['virtual_available'])
				falker_report2.append(execp)
		
		summary += '\n\n# Relatório de Pendências Tratadas:\n\n'
		if len(falker_report2) > 0:
			summary += '\n'.join(falker_report2)
		else:
			summary += '  Não há pendências tratadas.'
		
		summary += '\n\n# Relatório de Produtos Pendentes:\n\n'
		if len(falker_report) > 0:
			summary += '\n'.join(falker_report)
		else:
			summary += '  Não há pendências não tratadas.'

		summary += '\n'	
		request.create(cr, uid,
			{'name' : "Procurement calculation report.",
				'act_from' : user_id,
				'act_to' : user_id,
				'body': summary,
			})
	if use_new_cursor:
		cr.commit()
		cr.close()
	return {}
