# coding=utf-8
from osv import osv, fields

import pooler
import time
from mx import DateTime
import netsvc

# mrp_production (mrp.py:307)
#   def _get_warehouse_id
#   def action_compute
#   def action_confirm
class mrp_production(osv.osv):
	_inherit  = 'mrp.production'
	_columns = {
		'bom_id': fields.many2one('mrp.bom', 'Bill of Material', domain=[]),

		'location_stock_id': fields.many2one('stock.location', 'Origem', required=True),
		'location_dest_id': fields.many2one('stock.location', 'Destino', required=True),
		'location_src_id': fields.many2one('stock.location', 'Produção', required=True),	

		'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', required=True),	

		'origin': fields.char('Origin', size=64, readonly=True, select=True),

		# Não é obrigatório. Serve para a rastreabilidade do desfazer produção.
		'procurement_id': fields.many2one('mrp.procurement', 'Procurement', readonly=True),
	}

	def _get_warehouse_id(self, cr, uid, *args):
		cr.execute("SELECT id FROM stock_warehouse LIMIT 2")
		res = cr.fetchall()
		if len(res) == 1:
			return res[0][0]
		else:
			return False
				
	_defaults = {
		'warehouse_id': _get_warehouse_id,
	}

	def warehouse_id_change(self, cr, uid, ids, warehouse_id):
		if not warehouse_id:
			return {}
		else:	
			res = self.pool.get('stock.warehouse').read(cr, uid, [warehouse_id], ['lot_production_id','lot_destination_id','lot_origin_id'])[0]		
			src = res['lot_production_id'][0]
			dest = res['lot_destination_id'][0]
			stock = res['lot_origin_id'][0]
			result = {'location_src_id':src, 'location_stock_id':stock ,'location_dest_id':dest}			
		return {'value':result}

	#Verifica se todos os produtos são ignore. Neste caso ir direto para Ready To Produce
	def test_if_product(self, cr, uid, ids):
		res = True
		for production in self.browse(cr, uid, ids):
			if not production.product_lines:
				if not self.action_compute(cr, uid, [production.id]):
					res = False
			else:
				res = False
				for line in production.product_lines:
					if line.product_id.procure_method != 'ignore':
						res = True
						break
		return res
		
	def action_compute(self, cr, uid, ids, properties=[]):
		results = []
		for production in self.browse(cr, uid, ids):
			cr.execute('delete from mrp_production_product_line where production_id=%d', (production.id,))
			cr.execute('delete from mrp_production_workcenter_line where production_id=%d', (production.id,))
			bom_point = production.bom_id
			bom_id = production.bom_id.id
			if not bom_point:
				bom_id = self.pool.get('mrp.bom')._bom_find(cr, uid, production.product_id.id, production.product_uom.id, properties)
				self.write(cr, uid, [production.id], {'bom_id': bom_id})
				bom_point = self.pool.get('mrp.bom').browse(cr, uid, [bom_id])[0]

			if not bom_id:
				raise osv.except_osv('Error', "Couldn't find bill of material for product")

			if bom_point.routing_id and bom_point.routing_id.location_id:
				self.write(cr, uid, [production.id], {'location_src_id': bom_point.routing_id.location_id.id})

			
			factor = production.product_uom.factor / bom_point.product_uom.factor
			qty = factor * production.product_qty
			
			# Multiplica pelo número de produtos a serem produzidos. Cada conjunto de filhos produz UM produto pai.
			res = self.pool.get('mrp.bom')._bom_explode(cr, uid, bom_point, qty, properties)
			results = res[0]
			results2 = res[1]
			for line in results:
				# Multiplica pelo número de produtos a serem produzidos. Cada conjunto de filhos produz UM produto pai.
				line['production_id'] = production.id
				self.pool.get('mrp.production.product.line').create(cr, uid, line)
			for line in results2:
				line['production_id'] = production.id
				self.pool.get('mrp.production.workcenter.line').create(cr, uid, line)
		return len(results)		

	def action_confirm(self, cr, uid, ids):
		picking_id=False
		for production in self.browse(cr, uid, ids):
			if not production.product_lines:
				self.action_compute(cr, uid, [production.id])
				production = self.browse(cr, uid, [production.id])[0]
			picking_id = self.pool.get('stock.picking').create(cr, uid, {
				'origin': (production.origin or '').split(':')[0] +':'+production.name,
				'type': 'internal',
				'move_type': 'one',
				'state': 'auto',
				'auto_picking': True,
			})
			toconfirm = True

			source = production.product_id.product_tmpl_id.property_stock_production.id
			data = {
				'name':'PROD:'+production.name,
				'date_planned': production.date_planned,
				'product_id': production.product_id.id,
				'product_qty': production.product_qty,
				'product_uom': production.product_uom.id,
				'location_id': source,
				'location_dest_id': production.location_dest_id.id,
				'move_dest_id': production.move_prod_id.id,
				'state': 'waiting'
			}
			res_final_id = self.pool.get('stock.move').create(cr, uid, data)
			
			self.write(cr, uid, [production.id], {'move_created_ids': [(6, 'WTF', [res_final_id])]})
			moves = []
			
			for line in production.product_lines:

				# Se o procure_method for Ignore não deve criar Procurement nem Move.
				if line.product_id.procure_method == 'ignore':
					continue

				move_id=False
				newdate = production.date_planned
				if line.product_id.type in ('product', 'consu'):					
					res_dest_id = self.pool.get('stock.move').create(cr, uid, {
						'name':'PROD:'+production.name,
						'date_planned': production.date_planned,
						'product_id': line.product_id.id,
						'product_qty': line.product_qty,
						'product_uom': line.product_uom.id,
						'location_id': production.location_src_id.id,
						'location_dest_id': source,
						'move_dest_id': res_final_id,
						'state': 'waiting',
					})
					moves.append(res_dest_id)
					
					# Se houver Origem (location_stock_id) faz um move da origem,
					# caso não, faz um dummy-move
					if production.location_stock_id.id > 0:
						src = production.location_stock_id.id
					else:
						src = production.location_src_id.id
											
					move_id = self.pool.get('stock.move').create(cr, uid, {
						'name':'PROD:'+production.name,
						'picking_id':picking_id,
						'product_id': line.product_id.id,
						'product_qty': line.product_qty,
						'product_uom': line.product_uom.id,
						'date_planned': newdate,
						'move_dest_id': res_dest_id,
						'location_id': src,
						'location_dest_id': production.location_src_id.id,
						'state': 'waiting',
					})
				proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
					'name': (production.origin or '').split(':')[0] + ':' + production.name,
					'origin': (production.origin or '').split(':')[0] + ':' + production.name,
					'date_planned': newdate,
					'product_id': line.product_id.id,
					'product_qty': line.product_qty,
					'product_uom': line.product_uom.id,
					'location_id': production.location_src_id.id,
					'procure_method': line.product_id.procure_method,
					'move_id': move_id,
					'production_id': production.id, #Rastreabilidade do desfazer produção
				})
				wf_service = netsvc.LocalService("workflow")
				wf_service.trg_validate(uid, 'mrp.procurement', proc_id, 'button_confirm', cr)
			if toconfirm:
				wf_service = netsvc.LocalService("workflow")
				wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
			self.write(cr, uid, [production.id], {'picking_id':picking_id, 'move_lines': [(6,0,moves)], 'state':'confirmed'})
		return picking_id	

	# Não mexe no status do moves dos Insumos até que a produção seja concluído.
	# Mantendo assim os insumos na linha de produção.
	def action_in_production(self, cr, uid, ids):
		print 'action_in_production'
		move_ids = []
		for production in self.browse(cr, uid, ids):
			for res in production.move_lines:
				move_ids.append(res.id)
			if not production.date_start:
				self.write(cr, uid, [production.id],
						{'date_start': time.strftime('%Y-%m-%d %H:%M:%S')})
		#self.pool.get('stock.move').action_done(cr, uid, move_ids)
		self.write(cr, uid, ids, {'state': 'in_production'})
		return True
								
mrp_production()

# Inherited Class from mrp.py:615
class mrp_production_product_line(osv.osv):
	_inherit  = 'mrp.production.product.line'

	def _available_real(self, cr, uid, ids, name, arg, context):
		prod_ids = self.pool.get('mrp.production.product.line').read(cr, uid, ids, ['product_id'])
		res={}
		for count in range(len(prod_ids)):
			menge = self.pool.get('product.product').read(cr, uid, [prod_ids[count]['product_id'][0]], ['qty_available'])
			res[prod_ids[count]['id']]=menge[0]['qty_available']
		return res

	def _available_virtual(self, cr, uid, ids, name, arg, context):
		prod_ids = self.pool.get('mrp.production.product.line').read(cr, uid, ids, ['product_id'])
		res={}
		for count in range(len(prod_ids)):
			menge = self.pool.get('product.product').read(cr, uid, [prod_ids[count]['product_id'][0]], ['virtual_available'])
			res[prod_ids[count]['id']]=menge[0]['virtual_available']
		return res

	def _available(self, cr, uid, ids, name, arg, context):
		prod_ids = self.pool.get('mrp.production.product.line').read(cr, uid, ids, ['product_id','product_qty'])
		res={}
		for count in range(len(prod_ids)):
			menge = self.pool.get('product.product').read(cr, uid, [prod_ids[count]['product_id'][0]], ['virtual_available'])
			if prod_ids[count]['product_qty'] > menge[0]['virtual_available']:
				output = 'Falta: %(#).2F' % \
					{"#": prod_ids[count]['product_qty'] - menge[0]['virtual_available']}				
				res[prod_ids[count]['id']]=output.replace('.',',')

			else:
				res[prod_ids[count]['id']]='Sim'
		return res

	def _supply_method(self, cr, uid, ids, name, arg, context):
		prod_ids = self.pool.get('mrp.production.product.line').read(cr, uid, ids, ['product_id'])
		res={}
		for i in range(len(prod_ids)):
			menge = self.pool.get('product.product').read(cr, uid, [prod_ids[i]['product_id'][0]], ['supply_method'])
			if menge[0]['supply_method'] == 'produce':
				res[prod_ids[i]['id']] = 'Produce'
			else:
				res[prod_ids[i]['id']] = 'Buy'
		return res

	def _procure_method(self, cr, uid, ids, name, arg, context):
		prod_ids = self.pool.get('mrp.production.product.line').read(cr, uid, ids, ['product_id'])
		res={}
		for i in range(len(prod_ids)):
			menge = self.pool.get('product.product').read(cr, uid, [prod_ids[i]['product_id'][0]], ['procure_method'])
			if menge[0]['procure_method'] == 'make_to_stock':
				res[prod_ids[i]['id']] = 'Make to Stock'
			elif menge[0]['procure_method'] == 'make_to_order':
				res[prod_ids[i]['id']] = 'Make to Order'
			elif menge[0]['procure_method'] == 'ignore':
				res[prod_ids[i]['id']] = 'Ignore'
		return res
	
	_columns = {
		'available': fields.function(_available, method=True, type='char', string='Disponível'),
		'available_real': fields.function(_available_real, method=True, type='float', string='Estoque Real'),
		'available_virtual': fields.function(_available_virtual, method=True, type='float', string='Estoque Virtual'),
		'supply_method': fields.function(_supply_method, method=True, type='char', string='Supply Method'),
		'procure_method': fields.function(_procure_method, method=True, type='char', string='Procure Method'),
	}
mrp_production_product_line()

