# coding=utf-8
from osv import osv, fields

import time
import netsvc

class purchase_order(osv.osv):
	_inherit = 'purchase.order'

	# Preenchimento automático no campo: Purchase Order / Warehouse = 'Almoxarifado'	
	def _get_warehouse_id(self, cr, uid, *args):
		cr.execute("SELECT id FROM stock_warehouse LIMIT 2")
		res = cr.fetchall()
		if len(res) == 1:
			return res[0][0]
		else:
			return False

	_columns = {
		'origin': fields.char('Origin', size=64, readonly=True),
		'name': fields.char('Order Description', size=128, required=True, select=True),
		'date_next_action':fields.date('Data Próxima Ação', select=True),
		'next_action': fields.char('Próxima Ação', size=128, select=False),
		'date_preview': fields.date('Previsão de Entrega', select=True, states={'approved':[('readonly',True)],'done':[('readonly',True)]}),

		# Campo observação simples para fica na aba da frente das purchase.
		'obs': fields.char('Observação', size=256),

		# Não é obrigatório. Serve para a rastreabilidade do desfazer produção.
		'procurement_id': fields.many2one('mrp.procurement', 'Procurement', readonly=True),
	}

	_defaults = {
		'invoice_method': lambda *a: 'manual',
		'warehouse_id': _get_warehouse_id,
		'notes': lambda *a: '''------------------------------------------------------------
**  Informações sobre Pagamento
------------------------------------------------------------
Forma de Pagamento:
Prazo/Vencimento:

------------------------------------------------------------
** Informações sobre Frete
------------------------------------------------------------
Valor:
Forma de envio:
Pagamento do frete:

------------------------------------------------------------
** Informações Complementares / Irregularidades
------------------------------------------------------------


------------------------------------------------------------
** Próxima Ação
------------------------------------------------------------''',
	}

	# Desfazer Confirm Purchase
	def purchase_undo(self, cr, uid, ids, context={}):
		for purchase in self.browse(cr, uid, ids):
			if purchase.state == 'confirmed':
				self.write(cr, uid, [purchase.id],{
					'state': 'draft',
					'validator': False,
				})
				# Busca o id do wkf_instance do Purchase Order atual
				cr.execute("SELECT id FROM wkf_instance WHERE res_id=%i AND res_type='purchase.order'" % purchase.id)
				inst_id = cr.fetchone()[0]
				
				# Busca id do estado "draft" do workflow de "purchase.order"
				cr.execute("SELECT id FROM wkf_activity WHERE name='draft' AND wkf_id=(SELECT id FROM wkf WHERE osv='purchase.order')")
				draft_act_id = cr.fetchone()[0]

				# Busca id do estado "confirmed" do workflow de "purchase.order"
				cr.execute("SELECT id FROM wkf_activity WHERE name='confirmed' AND wkf_id=(SELECT id FROM wkf WHERE osv='purchase.order')")
				confirmed_act_id = cr.fetchone()[0]
				
				cr.execute("UPDATE wkf_workitem SET act_id=%i WHERE inst_id=%i AND act_id=%i" % (draft_act_id, inst_id, confirmed_act_id))
				cr.commit()
		return True


	# Desfazer Approved Purchase
	def purchase_approved_undo(self, cr, uid, ids, context={}):
		for purchase in self.browse(cr, uid, ids):

			if purchase.has_stockable_product(cr, uid, ids):
				should_has_packing = True
				cr.execute("SELECT id,state FROM stock_picking WHERE purchase_id=%d LIMIT 2", [purchase.id])
				if cr.rowcount == 0:
					raise UserWarning, 'Should be ONE packing with purchase_id = %d!' % purchase.id
				packing = cr.dictfetchone()
			else:
				should_has_packing = False

			if purchase.state == 'approved' and packing['state'] == 'assigned':
				self.write(cr, uid, [purchase.id],{
					'state': 'confirmed',
				})
				# Busca o id do wkf_instance do Purchase Order atual
				cr.execute("SELECT id FROM wkf_instance WHERE res_id=%i AND res_type='purchase.order'" % purchase.id)
				inst_id = cr.fetchone()[0]
				
				# Busca id do estado "confirmed" do workflow de "purchase.order"
				cr.execute("SELECT id FROM wkf_activity WHERE name='confirmed' AND wkf_id=(SELECT id FROM wkf WHERE osv='purchase.order')")
				draft_act_id = cr.fetchone()[0]

				# Busca id do estado "approved" do workflow de "purchase.order"
				cr.execute("SELECT id FROM wkf_activity WHERE name='picking' AND wkf_id=(SELECT id FROM wkf WHERE osv='purchase.order')")
				confirmed_act_id = cr.fetchone()[0]
				
				cr.execute("UPDATE wkf_workitem SET act_id=%i, subflow_id=NULL, state='complete' WHERE inst_id=%i AND act_id=%i" % (draft_act_id, inst_id, confirmed_act_id))


				# Apaga a Packing e o Move
				if should_has_packing:
					cr.execute("DELETE FROM wkf_instance WHERE res_id=%i AND res_type='stock.picking'" % packing['id'])
					cr.execute("DELETE FROM stock_move WHERE picking_id=%d", [packing['id']])
					cr.execute("DELETE FROM stock_picking WHERE id=%d", [packing['id']])

				cr.commit()
		return True
			
	# Preenchimento automático no campo: Purchase Order -> Approved By Supplier / Picking
	# Location Location=RECEBIMENTO; Final Location=Inusmos; Dest. Location=Insumos;
	# 	nas linhas da picking, de FORNECEDORES para RECEBIMENTO.
	def action_picking_create(self,cr, uid, ids, *args):
		picking_id = False
	
		for order in self.browse(cr, uid, ids):
						
			res = self.pool.get('stock.warehouse').read(cr, uid, [order.warehouse_id.id], ['lot_input_id','lot_origin_id'])[0]		
			location_id = res['lot_input_id'][0]
			location_dest_id = res['lot_origin_id'][0]

			loc_id = order.partner_id.property_stock_supplier.id
			istate = 'none'
			if order.invoice_method=='picking':
				istate = '2binvoiced'

			picking_id = self.pool.get('stock.picking').create(cr, uid, {
				'origin': order.name+((order.origin and (':'+order.origin)) or ''),
				'type': 'in',
				'address_id': order.dest_address_id.id or order.partner_address_id.id,
				'invoice_state': istate,
				'purchase_id': order.id,
				'location_id': location_id,
				'location_dest_id': location_dest_id,
				'loc_move_id': location_dest_id,
				'purchase_id': order.id, # Para rastreabilidade do desfazer
			})
			for order_line in order.order_line:
				if not order_line.product_id:
					continue
				if order_line.product_id.product_tmpl_id.type in ('product', 'consu'):
					dest = order.location_id.id
					self.pool.get('stock.move').create(cr, uid, {
						'name': 'PO:'+order_line.name,
						'product_id': order_line.product_id.id,
						'product_qty': order_line.product_qty,
						'product_uos_qty': order_line.product_qty,
						'product_uom': order_line.product_uom.id,
						'product_uos': order_line.product_uom.id,
						'date_planned': order_line.date_planned,
						'location_id': loc_id,
						'location_dest_id': dest,
						'picking_id': picking_id,
						'move_dest_id': order_line.move_dest_id.id,
						'state': 'assigned',
						'purchase_line_id': order_line.id,
					})
					if order_line.move_dest_id:
						self.pool.get('stock.move').write(cr, uid, [order_line.move_dest_id.id], {'location_id':order.location_id.id})
			wf_service = netsvc.LocalService("workflow")
			wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
		return picking_id

purchase_order()


class purchase_order_line(osv.osv):
	_inherit = 'purchase.order.line'

	# Cria campo função para visualizar o Partner Code Reference
	def _field_partner_code(self, cr, uid, ids, prop=None, *a):
		res = {}
		for line in self.browse(cr,uid,ids):
			partner_id = line.order_id.partner_id.id
			code_name = ''
			if line.product_id:
				for seller in line.product_id.seller_ids:
					if seller.name.id == partner_id:
						code_name = ''
						if seller.product_code:	
							code_name += '[%s] ' % seller.product_code
						if seller.product_name:	
							code_name += seller.product_name
						break
			res[line.id] = code_name
		return res
	
	_columns = {
		'name': fields.char('Description', size=256, required=True, select=True),
		'product_id': fields.many2one('product.product', 'Product', domain=[('purchase_ok','=',True)], change_default=True, select=True),		
		'partner_code': fields.function(_field_partner_code, method=True, string='Partner Reference', type="char"),	
		# Cria um número para cada item da compra
		'line_number': fields.integer('Item'),
	}

	def create(self, cr, uid, vals, context=None):
		# Cria um número para cada item da compra
		if vals['line_number'] == 0:
			order_id = str(vals['order_id'])
			cr.execute('SELECT line_number FROM purchase_order_line WHERE order_id=%s AND NOT line_number IS NULL ORDER BY line_number DESC LIMIT 1', [order_id])
			if cr.rowcount > 0:
				vals['line_number'] = cr.fetchone()[0] + 1
			else:
				vals['line_number'] = 1

		res = osv.osv.create(self, cr, uid, vals, context)
		return res

purchase_order_line()
