# MODULO COM FUNCOES UTEIS PARA MANIPULAR DICIONARIOS
# Criado por Carlos Klock - 20/nov/2008
# versao 2

# CDL = Create Dictionary of Lists
# Cria dicionario de listas, onde cada chave
# aponta para uma lista.
# keys: chaves que vao apontar para listas vazias.
# Retorna o dicionario que foi criado.
def cdl(keys):
	d = {}
	for k in keys:
		d[k] = []
	return d

# CDD = Create Dictionary of Dictionaries
# Cria dicionario de dicionarios, onde cada chave
# aponta para um dicionario.
# keys: chaves que vao apontar para dicionarios vazios.
# Retorna o dicionario que foi criado.
def cdd(keys):
	d = {}
	for k in keys:
		d[k] = {}
	return d

# FDL = Find in Dictionary of Lists
# Procura um elemento nas listas do dicionario
# de listas.
# d: o dicionario de listas.
# val: o valor a procurar dentro das listas.
# Retorna uma lista contendo as chaves aonde o
# valor foi encontrado.
def fdl(d, val):
	f = []
	for k in d:
		# Para cada lista do dicionario.
		l = d[k]
		if val in l:
			f.append(k)
	return f

# Coloca um valor no dicionario, no caminho especificado.
# O dicionario deve estar vazio ou conter dicionarios aninhados.
# d: o dicionario onde o valor deve ser colocado.
# keyPath: o caminho do dicionario, na forma de lista, onde
#  o valor deve ser colocado. Se o caminho nao existe ele eh criado.
# val: o valor a ser colocado no dicionario.
#
# == Exemplo ==
# d = {}
# print 'd antes =',d
# put(d, ('x',1,'z'), 5)
# print 'd depois =',d
#
# == Resultado do exemplo ==
# d antes = {}
# d depois = {'x': {1: {'z': 5}}}
#
def put(d, keyPath, val):
	if len(keyPath)==0:
		return
	if len(keyPath) > 1:
		for i in range(len(keyPath)-1):
			key = keyPath[i]
			if not d.has_key(key):
				d[key] = {}
			d = d[key]
		#print 'keyPath[i+1] =',keyPath[i+1]
		d[keyPath[i+1]] = val
	else:
		d[keyPath[0]] = val


# Retorna uma lista ordenada a partir das chaves
# do dicionario d.
# d: o dicionario de onde as chaves devem ser obtidas.
# asc: True indica que a ordenacao eh ascendente, False
#   indica ordenacao descendente.
def getSortIndex(d, asc=True):
	sortList = list(d.keys())
	sortList.sort()
	if asc==False:
		sortList.reverse()
	return sortList


# Cria tabela (lista de listas) a partir de um dicionario que contem as colunas da tabela.
# O dicionario deve conter uma chave 'header' que aponta para uma lista de nomes das colunas. 
# A partir dos nomes das colunas eh possivel obter a lista de valores da coluna.
# A ordem dos nomes das colunas na chave 'header' determina a ordem das colunas na tabela.
# Nenhuma coluna deve ter o nome 'header'.
# d_cols: {'header':lista_headers, 'nome_col1':lista_col1, 'nome_colN':lista_colN}
def createTableFromDic(d_cols):
	if type(d_cols)!=type({}):
		raise ValueError('d_cols deve ser dicionario!')
	if not d_cols.has_key('header'):
		raise ValueError('d_cols deve ter chave "header"!')
	if type(d_cols['header'])!=type([]):
		raise ValueError('d_cols["header"] deve conter uma lista com os nomes das colunas!')
		
	table_header = d_cols['header'] # lista com o nome das colunas
	ncols = len(table_header) # quantidade de colunas
	nrows = len(d_cols[table_header[0]]) # quantidade de linhas
	
	table = []
	table.append(table_header)	
	for row_i in range(nrows):
		row = []
		for col_i in range(ncols):
			col_name = table_header[col_i]
			col = d_cols[col_name] # lista com os elementos da coluna
			col_value = col[row_i]
			row.append(col_value)
		table.append(row)
	return table


#print cdl([])
#print cdl([1])
#print cdl([1,2])
#print
#print cdd([])
#print cdd([1])
#print cdd([1,2])
#print
#print fdl({}, 1)
#print fdl({'x':[1]}, 1)
#print fdl({'x':[]}, 1)
#print fdl({'x':[1,2]}, 2)
#print fdl({'x':[1]}, 2)
#print fdl({'x':[1], 'y':[5,6,2]}, 2)
#print
#d = {}
#put(d, [], 'x')
#print d
#d = {}
#put(d, [1], 'x')
#print d
#d = {}
#put(d, [1,2], 'x')
#print d
#d = {1:{1:{}}}
#put(d, [1,2], 'x')
#print d
#print getSortIndex({'a':1,'x':2,'h':3,'c':1})
#
#d_cols = {'header':['col1','col2']}
#d_cols['col1'] = [1,2,3,4,5]
#d_cols['col2'] = [5,4,3,2,1]
#table = createTableFromDic(d_cols)
#print d_cols
#print 'table:'
#for row in table:
#	print row
