#coding=utf-8

# Retorna dicionario com dados da main company:
# {nome, endereco, bairro, cidade, estado, cep, fone}
def getMainCompany(cr, uid):
	import pooler
	# Obtem dados da main company.
	cr.execute('SELECT partner_id FROM res_company WHERE id=(SELECT MAX(id) FROM res_company)')
	partner_id = cr.fetchone()[0]
	partner_model = pooler.get_pool(cr.dbname).get('res.partner')
	partner = partner_model.browse(cr, uid, [partner_id])[0]
	# DADOS DO REMETENTE
	dic_remetente = {}
	dic_remetente['nome'] = partner.name
	dic_remetente['endereco'] = partner.address[0].street
	dic_remetente['bairro'] = partner.address[0].street2
	dic_remetente['cidade'] = partner.address[0].city
	dic_remetente['estado'] = partner.address[0].state_id.code
	dic_remetente['cep'] = partner.address[0].zip
	dic_remetente['fone'] = partner.address[0].phone
	if not dic_remetente['nome'] or not dic_remetente['endereco'] or \
		not dic_remetente['bairro'] or not dic_remetente['cidade'] or \
		not dic_remetente['estado'] or not dic_remetente['cep'] or \
		not dic_remetente['fone']:
		raise ValueError('Os dados da main company (nome, endereco ou telefone) estao incompletos!')
	return dic_remetente

# Retorna dicionario com endereco/dados do partner (partner_address_id):
# {nome, ac, endereco, bairro, cep, cidade, estado}
def getPartnerAddress(cr, uid, partner_address_id, ac=None):
	if type(partner_address_id)!=int:
		raise ValueError('partner_address_id deve ser do tipo int!')
	import pooler
	# Obtem dados do partner.
	contatosdb = pooler.get_pool(cr.dbname).get('res.partner.address')
	contato = contatosdb.browse(cr, uid, [partner_address_id])[0]
	# nome, ac, endereco, cep, cidade, estado
	nome_empresa = contato.partner_id.name
	if not ac:
		ac = contato.name
	endereco = contato.street
	bairro = contato.street2
	cep = contato.zip
	cidade = contato.city
	if contato.state_id:
		estado = contato.state_id.code
	else:
		estado = False
	if not nome_empresa or not endereco or not bairro or not cep or not cidade or not estado:
		msg = 'Os dados do partner (nome ou endereco) estao incompletos!'
		raise ValueError(msg)
	return {'nome':nome_empresa, 'ac':ac, 'endereco':endereco, 'bairro':bairro, 'cep':cep, 'cidade':cidade, 'estado':estado}

# Retorna o modelo com o id especificado.
# model_id: id do modelo (registro da tabela).
# model_name: nome do modelo (tabela).
def getModel(cr, uid, model_id, model_name):
	if type(model_id)!=int:
		raise ValueError('model_id deve ser do tipo int!')
	if type(model_name)!=str:
		raise ValueError('model_id deve ser do tipo str!')
	import pooler
	p = pooler.get_pool(cr.dbname).get(model_name)
	return p.browse(cr, uid, [model_id])[0]

# Retorna uma lista de modelos com os ids especificados.
# model_ids: lista de ids.
# model_name: nome do modelo.
def getModels(cr, uid, model_ids, model_name):
	if type(model_ids)!=list:
		raise ValueError('model_ids deve ser do tipo list!')
	if type(model_name)!=str:
		raise ValueError('model_name deve ser do tipo str!')
	import pooler
	p = pooler.get_pool(cr.dbname).get(model_name)
	return p.browse(cr, uid, [model_ids])

# Retorna dados do pedido de compra/cotacao.
# Os dados sao retornados atraves de um dicionario:
# {num_pedido, data_pedido, previsao_entrega, order_lines, partner_address_id}
def getPurchaseOrderData(cr, uid, purchase_order_id, debug=False):
	if type(purchase_order_id)!=int:
		raise ValueError('purchase_order_id deve ser do tipo int!')
	import pooler
	purchasedb = pooler.get_pool(cr.dbname).get('purchase.order')
	purchase_model = purchasedb.browse(cr, uid, [purchase_order_id])[0]
	d_pedido = {}
	msg_ = ''
	has_error = False
	
	d_pedido['num_pedido'] = purchase_model.name
	d_pedido['data_pedido'] = purchase_model.date_order
	d_pedido['previsao_entrega'] = purchase_model.date_preview
	d_pedido['order_lines'] = purchase_model.order_line
	d_pedido['partner_address_id'] = purchase_model.partner_address_id.id
	d_pedido['total'] = purchase_model.amount_total
	if debug:
		print 'num_pedido:',d_pedido['num_pedido']
		print 'data_pedido:',d_pedido['data_pedido']
		print 'previsao_entrega:',d_pedido['previsao_entrega']
		print 'order_lines:',d_pedido['order_lines']
		print 'partner_address_id:',d_pedido['partner_address_id']
		print 'total:',d_pedido['total']
	
	if not d_pedido['num_pedido']:
		msg_ += 'num_pedido '
		has_error = True
	if not d_pedido['data_pedido']:
		msg_ += 'data_pedido '
		has_error = True
#	if not d_pedido['previsao_entrega']:
#		msg_ += 'previsao_entrega '
#		has_error = True
	if not d_pedido['order_lines']:
		msg_ += 'order_lines '
		has_error = True
	if not d_pedido['total']:
		msg_ += 'total '
		has_error = True
	if has_error:
		msg = 'Dados da purchase order estao incompletos!\n'
		msg += msg_
		raise ValueError(msg)
		
	return d_pedido

# Converte data em formato SQL para data em formato do Brasil.
# Ex: 2008-12-01 => 01-12-2008
def sqldate2brdate(sqldate):
	try:
		ano, mes, dia = sqldate.split('-')
		data = '%s/%s/%s' % (dia,mes,ano)
	except:
		raise ValueError('sqldate deve ser uma data em formato SQL!')
	return data

# Converte valor do campo para string. 
# Se valor eh false retorna string vazia.
def field2str(s):
	if s:
		return str(s)
	return ''

# Converte o valor v em uma string em reais.
def reais(v):
	try:
		value = float(v)
		import locale
		locale.setlocale(locale.LC_ALL, 'pt_BR.UTF-8')
		return 'R$ '+locale.format('%.2f',value,True,True)
	except:
		raise ValueError('v deve ser um numero!')

#print sqldate2brdate('2008-12-01')
#print reais('5.50')
