#coding=utf-8
import reportlab
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.units import inch, cm, mm
from reportlab.platypus import Paragraph, Frame, Table, BaseDocTemplate, PageTemplate, Spacer
from reportlab.lib import colors
from reportlab.lib.pagesizes import letter, A4
import rutils


# Registra fonte True Type.
def registerTTFont(fontName, fontFileName):
	import reportlab
	from reportlab.pdfbase import pdfmetrics
	from reportlab.pdfbase.ttfonts import TTFont
	reportlab.rl_config.warnOnMissingFontGlyphs = 0
	pdfmetrics.registerFont(TTFont(fontName, fontFileName))

# Funcao a ser chamada no evento 'onPage' da funcao createPdf().
# Esta funcao nao faz nada.
def _doNothing(canvas,document):
	pass

# Funcao utilitaria para criar PDF.
# fileName: nome ou caminho do arquivo PDF.
# pageSize: tamanho da pagina do arquivo PDF.
# story: lista de elementos do PDF (paragrafos, tabelas, spacers, etc)
# leftMargin: tamanho da margem esquerda.
# rightMargin: tamanho da margem direita.
# topMargin: tamanho da margem superior.
# bottomMargin: tamanho da margem inferior.
# title: titulo do PDF (aparece na janela). Quando nao for definido, o titulo
# eh o nome do arquivo.
# getPdfData: True retorna o conteudo do arquivo PDF.
# onPage: funcao a chamar quando uma pagina comeca a ser desenhada.
# onPageEnd: funcao a chamar quando uma pagina termina de ser desenhada.
def createPdf(fileName, pageSize, story,	leftMargin=2*cm, rightMargin=2*cm, 
	topMargin=2*cm, bottomMargin=2*cm, title=None, getPdfData=False, 
	onPage=_doNothing, onPageEnd=_doNothing):
	
	if type(fileName)!=str:
		raise ValueError('fileName must be a string!')
	if type(pageSize)!=tuple:
		raise ValueError('pageSize must be a tuple!')
	if type(story)!=list:
		raise ValueError('story must be a list!')
		
	import rml_utils
	frame_dic = rml_utils.frame_dict('frame1', pageSize=pageSize, leftMargin=leftMargin, rightMargin=rightMargin, 
		topMargin=topMargin, bottomMargin=bottomMargin)
	frame = Frame(id=frame_dic['id'], x1=frame_dic['x1'], y1=frame_dic['y1'], width=frame_dic['width'], height=frame_dic['height'])	
	
	if title==None:
		title = fileName
	template = PageTemplate(id='template1', frames=[frame], onPage=onPage, onPageEnd=onPageEnd)
	doc = BaseDocTemplate(filename=fileName, pagesize=pageSize, pageTemplates=[template], title=title, author='MAMS')
	doc.build(story)
	
	if getPdfData:
		try:
			f = open(fileName, 'rb')
			data = f.read()
			f.close()
			return data
		except:
			raise UserWarning('Nao foi possivel obter dados do arquivo! (getPdfData)')

# Funcao a ser chamada no evento 'onPage' da funcao createPdf()
# Esta funcao desenha um numero de pagina no rodape do PDF.
def drawPageNumber(canvas,document,
		text='pág',pageSize=A4,fontName="Helvetica",fontSize=8):
	if type(text)!=str:
		raise ValueError('text deve ser string!')
	if type(pageSize)!=tuple:
		raise ValueError('pageSize deve ser tuple!')
	if type(fontName)!=str:
		raise ValueError('fontName deve ser string!')
	if type(fontSize)!=int:
		raise ValueError('fontSize deve ser int!')
	pageNumber = canvas.getPageNumber()
	canvas.setFont(fontName, fontSize)
	x = float(pageSize[0])/2
	y = 1.6*cm
	canvas.drawCentredString(x, y, "%s %s" % (text, pageNumber))

# Cria um arquivo temporario.
# Esta funcao foi criada porque para retornar o conteudo
# do arquivo PDF no OpenERP eh necessario ler o arquivo
# PDF que foi gerado.
def tempFile():
	from tempfile import NamedTemporaryFile
	f = NamedTemporaryFile()
	return f.name

# Funcao para testar o Reportlab.
# Apenas para ver as caracteristicas e funcionamento.
def test(fileName):
	from reportlab.lib.styles import getSampleStyleSheet
	
	styles = getSampleStyleSheet()
	styleN = styles['Normal']
	styleH = styles['Heading1']
	story = []
	
	data = [['Top\nLeft', '', '02', '03', '04'],
		['', '', '12', '13', '14'],
		['20', '21', '22', 'Bottom\nRight', ''],
		['30', '31', '32', '', '']]
	t=Table(data,style=[
		('GRID',(0,0),(-1,-1),0.5,colors.grey),
		('BACKGROUND',(0,0),(1,1),colors.palegreen),
		('SPAN',(0,0),(1,1)),
		('BACKGROUND',(-2,-2),(-1,-1), colors.pink),
		('SPAN',(-2,-2),(-1,-1)),
		])
	
	story.append(Paragraph("This is a Heading",styleH))
	story.append(Paragraph("This is a paragraph in <i>Normal</i> style.", styleN))
	story.append(t)
	
	c = Canvas(fileName, pagesize=A4)
	f = Frame(inch, inch, 6*inch, 9*inch, showBoundary=1)
	f.addFromList(story,c)
	c.save()

# Testa uma tabela grande.
def testBigTable(fileName):
	style = [('GRID',(0,0),(-1,-1),1,colors.green), ('SPAN',(0,0),(1,0))]
	data = [['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],
		['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],['X','X'],]
	t = Table(data=data, style=style, repeatRows=1)
	createPdf(fileName, pageSize=letter, story=[Spacer(0,500), t], 
		leftMargin=2*cm, rightMargin=2*cm, topMargin=2*cm, bottomMargin=2*cm)
	print 'pdf criado'


# Cria uma tabela a partir de uma lista de conteúdos, que define o conteúdo de cada célula através
# de coordenadas.
# contents: conteúdo da tabela. É uma lista de tuplas que especifica o conteúdo das células, 
# onde tupla = (x, y, conteudo):
#    x é a coluna da célula, y é a linha e conteudo é o conteúdo da célula (texto ou lista de flowables).
# style: lista de estilos para usar na tabela.
# colWidths: largura de cada coluna.
# rowHeights: altura de cada linha.
# Retorna uma Table.
def criarTabela(contents, style, colWidths, rowHeights):
	if type(contents)!=list or type(style)!=list or type(colWidths)!=list or type(rowHeights)!=list:
		raise ValueError('Parametro deve ser uma lista!')
	table_data = rutils.matrix(ncols=len(colWidths), nrows=len(rowHeights), startVal='')
	# Insere conteudo na matriz.
	for tupla in contents:
		x = tupla[0]
		y = tupla[1]
		conteudo = tupla[2]
		table_data[y][x] = conteudo
	table = Table(table_data, style=style, colWidths=colWidths, rowHeights=rowHeights)
	return table

# Cria elementos de estilo para definir o 'padding' das células de uma tabela.
def tstylePadding(start=(0,0), stop=(-1,-1), lpad='1mm', rpad='1mm', tpad='1mm', bpad='1mm'):
	styles = []
	styles.append( ('LEFTPADDING',start,stop,lpad) )
	styles.append( ('RIGHTPADDING',start,stop,rpad) )
	styles.append( ('TOPPADDING',start,stop,tpad) )
	styles.append( ('BOTTOMPADDING',start,stop,bpad) )
	return styles


#test('/home/carlosklock/Área de Trabalho/mydoc.pdf')
#print 'pdf criado'
#
#data=[['Top\nLeft', '', '02', '03', '04'],
#	['', '', '12', '13', '14'],
#	['20', '21', '22', 'Bottom\nRight', ''],
#	['30', '31', '32', '', '']]
#t=Table(data,style=[
#	('GRID',(0,0),(-1,-1),0.5,colors.grey),
#	('BACKGROUND',(0,0),(1,1),colors.palegreen),
#	('SPAN',(0,0),(1,1)),
#	('BACKGROUND',(-2,-2),(-1,-1), colors.pink),
#	('SPAN',(-2,-2),(-1,-1)),
#	])
#
#testBigTable('/home/carlosklock/Área de Trabalho/mydoc.pdf')
#
#print tempFile()
