# -*- coding: utf-8 -*-
from report.interface import report_rml
from report.interface import toxml
import pooler
import rutils
import pdf_utils
from reportlab.lib.units import inch, cm, mm
from reportlab.platypus import Paragraph, Frame, Table, Spacer
from reportlab.pdfgen.canvas import Canvas	
from reportlab.lib import colors
from reportlab.lib.pagesizes import letter, A4
from reportlab.lib.styles import ParagraphStyle


# Cria PDF para geracao de etiquetas.
# dic_etiquetas: num_etiqueta => dados_etiqueta
#   dados_etiqueta é dicionário com as seguintes chaves:
#   nome_empresa, a_c, rua, bairro, cep, cidade, uf
def criarPdf(dic_etiquetas):
	#nome_empresa='NOME DA EMPRESA', a_c='A/C NOME DO RESPONSÁVEL', rua='Rua', 
	#bairro='Bairro', cep='CEP', cidade='Cidade', uf='UF', etiquetas=[1,2,3,4,5,6]
	contents = []
	
	for num_etiqueta in dic_etiquetas:
		try:
			etiqueta = dic_etiquetas[num_etiqueta]
			nome_empresa = etiqueta['nome_empresa']
			a_c = etiqueta['a_c']
			rua = etiqueta['rua']
			bairro = etiqueta['bairro']
			cep = etiqueta['cep']
			cidade = etiqueta['cidade']
			uf = etiqueta['uf']
		except:
			print 'Dicionário vazio ou inválido!'
			return

		pdf_utils.registerTTFont('LucidaSans', 'lsans.ttf')
		pdf_utils.registerTTFont('LucidaSans-bi', 'lsansbi.ttf')
		styleTitulo = pstyle(name='Titulo', fontName='LucidaSans-bi')
		styleNormal = pstyle(name='Normal', fontName='LucidaSans')
		
		conteudo = []
		conteudo.append(Paragraph('Destinatário:', styleTitulo))
		conteudo.append(Spacer(0, 0.6*cm))
		conteudo.append(Paragraph(nome_empresa, styleNormal))
		conteudo.append(Spacer(0, 0.6*cm))
		conteudo.append(Paragraph('A/C  '+str(a_c), styleNormal))
		conteudo.append(Spacer(0, 0.6*cm))
		conteudo.append(Paragraph(rua, styleNormal))
		conteudo.append(Paragraph(bairro, styleNormal))
		conteudo.append(Paragraph(cep, styleNormal))
		conteudo.append(Paragraph(str(cidade)+' - '+str(uf), styleNormal))
		
		if num_etiqueta==1:
			contents.append((0,0,conteudo))
		elif num_etiqueta==2:
			contents.append((1,0,conteudo))
		elif num_etiqueta==3:
			contents.append((0,1,conteudo))
		elif num_etiqueta==4:
			contents.append((1,1,conteudo))
		elif num_etiqueta==5:
			contents.append((0,2,conteudo))
		elif num_etiqueta==6:
			contents.append((1,2,conteudo))
	## END FOR ##
	
	styleLeft = tstyleEtiquetas(background='white', lpad=5*mm, rpad=7.5*mm, tpad=5*mm, bpad=5*mm, start=(0,0), stop=(0,-1))
	styleRight = tstyleEtiquetas(background='white', lpad=7.5*mm, rpad=5*mm, tpad=5*mm, bpad=5*mm, start=(1,0), stop=(1,-1))
	styles = styleLeft + styleRight
	
	table = pdf_utils.criarTabela(contents=contents, style=styles, colWidths=[101.6*mm]*2, rowHeights=[84.67*mm]*3)
	
	fileName = pdf_utils.tempFile()
	return pdf_utils.createPdf(fileName=fileName, title='etiquetas_correspondencia.pdf',
		pageSize=letter, story=[table], 
		leftMargin=4*mm, rightMargin=4*mm, topMargin=1*cm, bottomMargin=1*cm,
		getPdfData=True)


# Define a formatacao/estilo das etiquetas.
# background: cor de fundo.
# lpad: tamanho da margem esquerda.
# rpad: tamanho da margem direita.
# tpad: tamanho da margem superior.
# bpad: tamanho da margem inferior.
def tstyleEtiquetas(background='white', 
	lpad=3*mm, rpad=3*mm, tpad=3*mm, bpad=3*mm, 
	start=(0,0), stop=(-1,-1), drawGrid=False):
		
	styles = []
	styles.append(('ALIGNMENT',(0,0),(-1,-1),'CENTER'))
	styles.append(('VALIGN',(0,0),(-1,-1),'MIDDLE'))
	styles.append(('BACKGROUND',(0,0),(-1,-1),background))
	
	styles.append(('LEFTPADDING',start,stop,lpad))
	styles.append(('RIGHTPADDING',start,stop,rpad))
	styles.append(('TOPPADDING',start,stop,tpad))
	styles.append(('BOTTOMPADDING',start,stop,bpad))
	
	# GRID PARA TESTE
	if drawGrid:
		styles.append(('GRID',(0,0),(-1,-1),1,'red'))
	
	return styles

# Define o estilo de um paragrafo.
def pstyle(name, fontName, fontSize=14, textColor='black', 
		spaceBefore=0, spaceAfter=0, leading=16, alignment=0):
	# Alignment: 0=left, 1=center, 2=right, 4=justify
	return ParagraphStyle(name=name, fontName=fontName, fontSize=fontSize, 
		textColor=textColor, spaceBefore=spaceBefore, spaceAfter=spaceAfter,
		leading=leading, alignment=alignment)



class report_custom(report_rml):
	def create_xml(self, cr, uid, ids, datas, context=None):
		#print 'ids:',ids
		#print 'datas:',datas
		self.cr = cr
		self.uid = uid
		self.ids = ids
		self.datas = datas
		return ''

	def create_rml(self, cr, xml, uid, context=None):
		rml = ''
		return rml

	def create_pdf(self, xml):
		cr = self.cr
		uid = self.uid
		ids = self.ids
		datas = self.datas
		contatosdb = pooler.get_pool(cr.dbname).get('res.partner.address')
		#print 'contatosdb:',contatosdb
		dic_etiquetas = {}

		# Obtem os dados para criar as etiquetas.
		for key in datas['form']:
			if key.find('contato')!=0 or not key[-1].isdigit():
				continue
			#print 'key:',key
			#print 'datas form:',datas['form']
			contato_id = datas['form'][key]
			#print 'contato_id:',contato_id
			
			# Numero da etiqueta eh extraido da
			# string contatoN, onde N eh o numero
			# da etiqueta.
			num_etiqueta = int(key[7:])
			
			if contato_id != False:
				try:
					contato = contatosdb.browse(cr, uid, [contato_id])[0]
					#print 'contato:',contato
					nome_empresa = contato.partner_id.name
					if datas['form']['ac'+str(num_etiqueta)]:
						a_c = datas['form']['ac'+str(num_etiqueta)]
					else:
						a_c = contato.name
					rua = contato.street
					bairro = contato.street2
					cep = contato.zip
					cidade = contato.city
					uf = contato.state_id.code
				except:
					print 'O contato selecionado tem campos não preenchidos!'
					return
				
				#print 'dados partner:'
				#print 'nome_empresa:',nome_empresa,'a_c:',a_c,'rua:',rua,'bairro:',bairro,'cep:',cep,'cidade:',cidade,'uf:',uf
				dic_etiquetas[num_etiqueta] = {}
				dic_etiquetas[num_etiqueta]['nome_empresa'] = nome_empresa
				dic_etiquetas[num_etiqueta]['a_c'] = a_c
				dic_etiquetas[num_etiqueta]['rua'] = rua
				dic_etiquetas[num_etiqueta]['bairro'] = bairro
				dic_etiquetas[num_etiqueta]['cep'] = cep
				dic_etiquetas[num_etiqueta]['cidade'] = cidade
				dic_etiquetas[num_etiqueta]['uf'] = uf	

		return criarPdf(dic_etiquetas)

report_custom('report.contatos.etiquetas', 'res.partner.address', '', '')
