# -*- coding: utf-8 -*-
##############################################
# Este modulo cria um relatorio para a 
# geracao de etiquetas de produtos.
##############################################
# Autor: Carlos Klock
##############################################
from report.interface import report_rml
from report.interface import toxml
import pooler
import rutils
import pdf_utils
from reportlab.lib.units import inch, cm, mm
from reportlab.platypus import Paragraph, Frame, Table, Spacer
from reportlab.pdfgen.canvas import Canvas	
from reportlab.lib import colors
from reportlab.lib.pagesizes import letter, A4
from reportlab.lib.styles import ParagraphStyle


# Cria PDF para geracao de etiquetas.
# dic_etiquetas: num_etiqueta => dados_etiqueta
def criarPdf(dic_etiquetas):	
	dic_mapa = mapearEtiquetas(dic_etiquetas)
	contents = []
	
	for num_etiqueta in dic_etiquetas:
		try:
			etiqueta = dic_etiquetas[num_etiqueta]
			name = etiqueta['name']
			code = etiqueta['code']
		except:
			print 'Dicionário vazio ou inválido!'
			return

		styleNormal = pstyle(name='Normal', fontName='Helvetica')
		
		conteudo = []
		conteudo.append(Paragraph(name, styleNormal))
		conteudo.append(Paragraph(code, styleNormal))
		
		coords = dic_mapa[num_etiqueta]
		dados = (coords[0], coords[1], conteudo)
		contents.append(dados)
		
	table = pdf_utils.criarTabela(contents=contents, style=tstyleEtiquetas(), 
		colWidths=[66.7*mm,3*mm,66.7*mm,3*mm,66.7*mm], rowHeights=[25.4*mm]*10)
	
	fileName = pdf_utils.tempFile()
	return pdf_utils.createPdf(fileName=fileName, title='etiquetas_produtos.pdf',
		pageSize=letter, story=[table],	
		leftMargin=5*mm, rightMargin=4*mm, topMargin=11*mm, bottomMargin=10*mm,
		getPdfData=True)

	
# Mapeia numero da etiqueta para uma celula da tabela
# de etiquetas.
# Retorna dicionario: num_etiqueta => coordenada da celula
def mapearEtiquetas(dic_etiquetas):
	num_etiquetas = list(dic_etiquetas.keys())
	num_etiquetas.sort()
	n = 0
	col_index = [0,2,4]
	nrows = 10
	dic_mapa = {}
	for y in range(nrows):
		for x in col_index:
			n += 1
			dic_mapa[n] = (x,y)
	return dic_mapa
	
# Define o estilo de um paragrafo.
def pstyle(name, fontName, fontSize=10, textColor='black', 
		spaceBefore=0, spaceAfter=0, leading=11, alignment=1):
	# Alignment: 0=left, 1=center, 2=right, 4=justify
	return ParagraphStyle(name=name, fontName=fontName, fontSize=fontSize, 
		textColor=textColor, spaceBefore=spaceBefore, spaceAfter=spaceAfter,
		leading=leading, alignment=alignment)

# Define a formatacao/estilo das etiquetas.
# lpad: tamanho da margem esquerda.
# rpad: tamanho da margem direita.
# tpad: tamanho da margem superior.
# bpad: tamanho da margem inferior.
def tstyleEtiquetas(lpad=2*mm, rpad=2*mm, tpad=2*mm, bpad=2*mm, 
	start=(0,0), stop=(-1,-1), drawGrid=False):
		
	styles = []
	styles.append(('ALIGNMENT',(0,0),(-1,-1),'CENTER'))
	styles.append(('VALIGN',(0,0),(-1,-1),'MIDDLE'))
	styles.append(('LEFTPADDING',start,stop,lpad))
	styles.append(('RIGHTPADDING',start,stop,rpad))
	styles.append(('TOPPADDING',start,stop,tpad))
	styles.append(('BOTTOMPADDING',start,stop,bpad))
	
	if drawGrid:
		styles.append(('GRID',(0,0),(-1,-1),0.5,'red'))
	
	return styles


class report_custom(report_rml):

	def create_xml(self, cr, uid, ids, datas, context=None):
		self.cr = cr
		self.uid = uid
		self.ids = ids
		self.datas = datas
		return ''

	def create_rml(self, cr, xml, uid, context=None):
		rml = ''
		return rml

	def create_pdf(self, xml):
		cr = self.cr
		uid = self.uid
		ids = self.ids
		datas = self.datas
		
		produtosdb = pooler.get_pool(cr.dbname).get('product.product')
		dic_etiquetas = {}

		# Obtem os dados para criar as etiquetas.
		for key in datas['form']:
			if key.find('produto')!=0 or not key[-1].isdigit():
				continue

			produto_id = datas['form'][key]
			
			# Numero da etiqueta eh extraido da
			# string produtoN, onde N eh o numero
			# da etiqueta.
			num_etiqueta = int(key[7:])
			
			if produto_id != False:
				try:
					produto = produtosdb.browse(cr, uid, [produto_id])[0]
					name = produto.product_tmpl_id.name
					code = produto.default_code
				except:
					print 'O produto selecionado tem campos não preenchidos!'
					return
				
				dic_etiquetas[num_etiqueta] = {}
				dic_etiquetas[num_etiqueta]['name'] = name
				dic_etiquetas[num_etiqueta]['code'] = code
		
		return criarPdf(dic_etiquetas)

report_custom('report.produtos.etiquetas', 'product.product', '', '')


#dic_etiquetas = {}
#dic_etiquetas[1] = {'name':'Produto X', 'code':'123.dfgs.456789-9908'}
#dic_etiquetas[10] = {'name':'Um produto com um nome muito longo ajdfkfjlkdjfldkfjlak djfldjflkdjfl kdjflkadjfldkjfkldfj', 'code':'123.dfgs.456789-9908'}
#criarPdf(dic_etiquetas)
#print 'pdf criado'
