# -*- coding: utf-8 -*-
##############################################
# Autor: Carlos Klock
##############################################
from report.interface import report_rml
from report.interface import toxml
import pooler
import wizard
import rutils
import pdf_utils
from reportlab.lib.units import inch, cm, mm
from reportlab.platypus import Paragraph, Frame, Table, Spacer, Image
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib import colors
from reportlab.lib.pagesizes import letter, A4
from reportlab.lib.styles import ParagraphStyle

def criarPdf(dic_destinatario, dic_remetente, num_nota_fiscal, produtos, observacoes, alertas):
	nome_destinatario = dic_destinatario['nome']
	ac = dic_destinatario['ac']
	endereco = dic_destinatario['endereco']
	cep = dic_destinatario['cep']
	cidade = dic_destinatario['cidade']
	estado = dic_destinatario['estado']
	
	pdf_utils.registerTTFont('LucidaSans', 'lsans.ttf')
	pdf_utils.registerTTFont('LucidaSans-b', 'lsansb.ttf')
	pdf_utils.registerTTFont('LucidaSans-i', 'lsansi.ttf')
	pdf_utils.registerTTFont('LucidaSans-bi', 'lsansbi.ttf')
	
	t_dest = criarTabelaDestinatario(nome_destinatario, ac, endereco, cep, cidade, estado)
	t_rem = criarTabelaRemetente(dic_remetente['nome'], dic_remetente['endereco'], 
		dic_remetente['bairro'], dic_remetente['cidade'], dic_remetente['estado'], 
		dic_remetente['cep'], dic_remetente['fone'])
	t_prod1, t_prod2 = criarTabelasProdutos(num_nota_fiscal, produtos)
	t_obs = criarTabelaObservacoes(observacoes, alertas)
	
	story = []
	story.append(t_dest)
	story.append(Spacer(0, 0.8*cm))
	story.append(t_rem)
	story.append(Spacer(0, 0.8*cm))
	story.append(t_prod1)
	story.append(t_prod2)
	story.append(Spacer(0, 0.8*cm))
	story.append(t_obs)
	
	return pdf_utils.createPdf(fileName=pdf_utils.tempFile(), title='identificacao_remessa.pdf', 
		pageSize=A4, story=story, 
		leftMargin=1.27*cm, rightMargin=1.27*cm, topMargin=1.27*cm, bottomMargin=1.27*cm,
		getPdfData=True)

# Cria tabela de destinatario.
# A tabela eh construida com 3 linhas e 3 colunas. Eh usado span para unir celulas.
def criarTabelaDestinatario(nome_destinatario, ac, endereco, cep, cidade, estado):
	pstyle_dest = pstyle(name='dest',fontName='LucidaSans',fontSize=14,leading=15)
	p_dest = Paragraph(nome_destinatario, pstyle_dest)
	p_ac = Paragraph(ac, pstyle_dest)
	
	contents = []
	contents.append(['Destinatário:', [p_dest,Spacer(0,0.5*cm),p_ac], [p_dest,Spacer(0,0.5*cm),p_ac]])
	contents.append(['Endereço:', endereco, endereco])
	contents.append(['CEP: %s'%(cep), 'Cidade: %s'%(cidade), 'Estado: %s'%(estado)])
	
	styles = []
	styles.append( ('FONTNAME',(0,0),(-1,-1),'LucidaSans') )
	styles.append( ('FONTSIZE',(0,0),(-1,-1),10) )
	#styles.append( ('FONTSIZE',(1,0),(2,0),14) )
	styles.append( ('VALIGN',(0,0),(-1,-1),'MIDDLE') )
	styles.append( ('ALIGNMENT',(1,0),(2,0),'CENTER') )
	styles.append( ('GRID',(0,0),(-1,-1),0.5,'black') )
	styles.append( ('SPAN',(1,0),(2,0)) )
	styles.append( ('SPAN',(1,1),(2,1)) )
	
	t_dest = Table(data=contents, style=styles, 
		colWidths=[4.5*cm, 8*cm, 4.5*cm], rowHeights=[3*cm, 1*cm, 1*cm])
	return t_dest
	
def criarTabelaRemetente(nome, endereco, bairro, cidade, estado, cep, fone):
	ps_nome = pstyle(name='nome',fontName='LucidaSans-b',fontSize=10,leading=14)
	ps_corpo = pstyle(name='corpo',fontName='LucidaSans',fontSize=8,leading=12)
	pnome = Paragraph(nome, ps_nome)
	pendereco = Paragraph(endereco, ps_corpo)
	endereco2 = ' - '.join([bairro, cidade, estado, cep])
	pendereco2 = Paragraph(endereco2, ps_corpo)
	pfone = Paragraph(fone, ps_corpo)
	image = Image('addons/falker/report/falker_logo_pequeno.gif')
	#image.drawWidth = 125
	#image.drawHeight = 66
	
	data = []
	data.append(['Remetente:', ''])
	data.append([image, [pnome, pendereco, pendereco2, pfone]])
	
	styles = []
	styles.append( ('FONTNAME',(0,0),(-1,-1),'LucidaSans') )
	styles.append( ('FONTSIZE',(0,0),(-1,-1),10) )
	styles.append( ('VALIGN',(0,0),(-1,-1),'MIDDLE') )
	styles.append( ('ALIGNMENT',(0,0),(-1,-1),'CENTER') )
	styles.append( ('ALIGNMENT',(0,0),(0,0),'LEFT') )
	styles.append( ('GRID',(0,0),(-1,-1),0.5,'black') )
	
	table = Table(data=data, style=styles,
		colWidths=[6*cm, 11*cm], rowHeights=[1*cm, 2.5*cm])
	return table

def criarTabelasProdutos(num_nota_fiscal, produtos):	
	# matriz de dados da tabela 1 (lista de linhas da tabela)
	data = []
	data.append([Paragraph('Número da Nota Fiscal',PNORMAL), Paragraph(num_nota_fiscal,PNORMAL)])
	data.append(['Conteúdo', None])
		
	# matriz de dados da tabela 2 (lista de linhas da tabela)
	data_prod = []
	data_prod.append([Paragraph('Qtd',PNEG), Paragraph('Descrição',PNEG)])
	for produto in produtos:
		desc = produto[0]
		qtd = produto[1]
		data_prod.append([Paragraph(str(qtd),PNORMAL), Paragraph(str(desc),PDESC)])
	
	styles = []
	styles.append( ('FONTNAME',(0,0),(-1,-1),'LucidaSans-b') )
	styles.append( ('FONTSIZE',(0,0),(-1,-1),10) )
	styles.append( ('VALIGN',(0,0),(-1,-1),'MIDDLE') )
	styles.append( ('ALIGNMENT',(0,0),(-1,-1),'CENTER') )
	#styles.append( ('ALIGNMENT',(0,1),(-1,1),'LEFT') )
	styles.append( ('GRID',(0,0),(-1,-1),0.5,'black') )
	styles.append( ('SPAN',(0,1),(-1,1)) )
#	styles.append( ('SPAN',(0,1),(-1,1)) )	
#	styles.append( ('SPAN',(0,2),(-1,2)) )	
	#styles += pdf_utils.tstylePadding((0,2),(-1,2),0,0,0,0)
	
	table = Table(data=data, style=styles, colWidths=[6.5*cm, 10.5*cm])
	table2 = Table(data=data_prod, style=tstyleDefault(drawGrid=True), colWidths=[2*cm, 15*cm])
	return [table,table2]
	
def criarTabelaObservacoes(observacoes, alertas):
	ptitulo1 = Paragraph('Observações', PNEG)
	pobs = Paragraph(str(observacoes), PNORMAL)
	palertas = Paragraph(str(alertas), pstyle(name='p',fontName='LucidaSans-b',fontSize=30,textColor='red',))
	data = [] # Dados da tabela
	data.append([ptitulo1])
	if observacoes:
		data.append([pobs])
	data.append([palertas])

	styles = tstyleDefault(drawGrid=True)
	styles.append( ('TOPPADDING',(0,0),(-1,-1),2*mm) )
	table = Table(data=data, style=styles, colWidths=[17*cm])
	return table

# Define o estilo de um paragrafo.
# alignment: 0=left, 1=center, 2=right, 4=justify
def pstyle(name, fontName, fontSize=10, textColor='black', 
		spaceBefore=0, spaceAfter=0, leading=None, alignment=1):
	if leading==None:
		leading = fontSize
	return ParagraphStyle(name=name, fontName=fontName, fontSize=fontSize, 
		textColor=textColor, spaceBefore=spaceBefore, spaceAfter=spaceAfter,
		leading=leading, alignment=alignment)

def tstyleDefault(drawGrid=False):
	styles = []
	styles.append( ('FONTNAME',(0,0),(-1,-1),'LucidaSans') )
	styles.append( ('FONTSIZE',(0,0),(-1,-1),10) )
	styles.append( ('VALIGN',(0,0),(-1,-1),'MIDDLE') )
	if drawGrid:
		styles.append( ('GRID',(0,0),(-1,-1),0.5,'black') )
	return styles
	
PNEG = pstyle(name='negrito', fontName='LucidaSans-b', fontSize=10)
PNORMAL = pstyle(name='normal', fontName='LucidaSans', fontSize=10)
PDESC = pstyle(name='normal', fontName='LucidaSans', fontSize=10, alignment=0, leading=12)

class report_custom(report_rml):

	def create_xml(self, cr, uid, ids, datas, context=None):
		self.cr = cr
		self.uid = uid
		self.ids = ids
		self.datas = datas
		return ''

	def create_rml(self, cr, xml, uid, context=None):
		rml = ''
		return rml

	def create_pdf(self, xml):
		cr = self.cr
		uid = self.uid
		ids = self.ids
		datas = self.datas
		#print 'report datas:',datas
		form = datas['form']
		
		# DADOS DO DESTINATARIO
		contato_id = form['contato']
		contatosdb = pooler.get_pool(cr.dbname).get('res.partner.address')
		contato = contatosdb.browse(cr, uid, [contato_id])[0]
		# nome, ac, endereco, cep, cidade, estado
		nome_empresa = contato.partner_id.name
		ac = form['ac']
		if not ac:
			ac = contato.name
		ac = 'A/C  '+ac
		endereco = contato.street
		#bairro = contato.street2
		cep = contato.zip
		cidade = contato.city
		if contato.state_id:
			estado = contato.state_id.code
		else:
			estado = False
		if not endereco or not cep or not cidade or not estado:
			msg = 'O endereco do contato esta incompleto!'
			print msg
			return
		dic_dest = {'nome':nome_empresa, 'ac':ac, 'endereco':endereco, 'cep':cep, 'cidade':cidade, 'estado':estado}

		try:
			# Obtem dados da main company.
			cr.execute('SELECT partner_id FROM res_company WHERE id=(SELECT MAX(id) FROM res_company)')
			partner_id = cr.fetchone()[0]
			partner_model = pooler.get_pool(cr.dbname).get('res.partner')
			partner = partner_model.browse(cr, uid, [partner_id])[0]
			# DADOS DO REMETENTE
			dic_remetente = {}
			dic_remetente['nome'] = partner.name
			dic_remetente['endereco'] = partner.address[0].street
			dic_remetente['bairro'] = partner.address[0].street2
			dic_remetente['cidade'] = partner.address[0].city
			dic_remetente['estado'] = partner.address[0].state_id.code
			dic_remetente['cep'] = partner.address[0].zip
			dic_remetente['fone'] = partner.address[0].phone
	#		print 'partner id:',partner_id
	#		print 'partner name:',partner.name
		except:
			msg = 'O endereco do remetente esta incompleto!'
			print msg
			return			

#		dic_remetente['nome'] = 'Falker Automação Agrícola Ltda.'
#		dic_remetente['endereco'] = 'Rua Washington Luiz, 675 / 1112'
#		dic_remetente['bairro'] = 'Centro'
#		dic_remetente['cidade'] = 'Porto Alegre'
#		dic_remetente['estado'] = 'RS'
#		dic_remetente['cep'] = 'CEP 90.010-460'
#		dic_remetente['fone'] = 'Fone (51) 3019-3730'
		
		# num_nota_fiscal, observacoes, alertas, produtos, qtd, num_serie
		num_nota = form['num_nota']
		observacoes = form['observacoes']
		alertas = form['alertas']
		if not observacoes:
			observacoes = ''
		if not alertas:
			alertas = ''
		
		produtos = []
		for key in form:
			if 'produto' in key and key[-1].isdigit():
				i = key[7:]
				#print 'key:',key
				#print 'i:',i
				produto = form['produto'+i]
				qtd = form['qtd'+i]
				num_serie = ''
				if form['num_serie'+i]:
					num_serie = '<br/><br/>N.Série ' + form['num_serie'+i]
				produtos.append( (produto+num_serie, qtd) )
				
		return criarPdf(dic_dest, dic_remetente, num_nota, produtos, observacoes, alertas)

report_custom('report.product.identificacao_remessa', 'product.product', '', '')


##nome_destinatario, ac, endereco, cep, cidade, estado, num_nota_fiscal, produtos, observacoes
#dic_destinatario = {}
#dic_destinatario['nome'] = 'Uma empresa com um NOME EXTREMAMENTE GRANDE PARA TESTAR O ESPACAMENTO'
#dic_destinatario['ac'] = 'A/C Cristovão Buarque de Holanda'
#dic_destinatario['endereco'] = 'Rua x, numero y'
#dic_destinatario['cep'] = '90.000-000'
#dic_destinatario['cidade'] = 'Porto Alegre'
#dic_destinatario['estado'] = 'RS'
#num_nota_fiscal = '64657adf'
##produtos = [('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1), ('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),]
#produtos = [('Produto X', 2), ('Produto Y', 1), ('Produto Z', 1),]
#observacoes = 'ATENÇÃO: CONTEÚDO FRÁGIL!'
#dic_remetente = {}
#dic_remetente['nome'] = 'Falker Automação Agrícola Ltda.'
#dic_remetente['endereco'] = 'Rua Washington Luiz, 675 / 1112'
#dic_remetente['bairro'] = 'Centro'
#dic_remetente['cidade'] = 'Porto Alegre'
#dic_remetente['estado'] = 'RS'
#dic_remetente['cep'] = 'CEP 90.010-460'
#dic_remetente['fone'] = 'Fone (51) 3019-3730'
#
#criarPdf(dic_destinatario, dic_remetente, num_nota_fiscal, produtos, observacoes)
#print 'pdf criado'
