#coding=utf-8
import pdf_utils
import openerp_utils
from reportlab.lib.units import inch, cm, mm
from reportlab.platypus import Paragraph, Frame, Table, Spacer, Image
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib import colors
from reportlab.lib.pagesizes import letter, A4
from reportlab.lib.styles import ParagraphStyle

# GERADOR DE PDF PARA ORDEM DE COMPRA E PEDIDO DE COTACAO.

d_company = {}

# Cria PDF a partir dos dados fornecidos.
# Retorna o PDF.
# 
# partner_address_id: id do partner do destinatario.
# d_pedido: dicionario de informacoes do pedido.
# produtos_lines: lista de dicionarios, onde cada 
#   dicionario eh uma linha da tabela de produtos do pedido.
def criarPdf(cr, uid, report_name, d_pedido, produtos_lines):
	if type(produtos_lines)!=list:
		raise ValueError('produtos_lines deve ser uma lista de dicionarios!')
	if type(d_pedido)!=dict:
		raise ValueError('d_pedido deve ser um dicionario!')
	
	# Registra conjunto de fontes LucidaSans.
	pdf_utils.registerTTFont('LucidaSans', 'lsans.ttf')
	pdf_utils.registerTTFont('LucidaSans-b', 'lsansb.ttf')
	pdf_utils.registerTTFont('LucidaSans-i', 'lsansi.ttf')
	pdf_utils.registerTTFont('LucidaSans-bi', 'lsansbi.ttf')
	
	# MAIN COMPANY (Falker nesse caso).
	from openerp_utils import getMainCompany
	global d_company
	d_company = getMainCompany(cr, uid)
	
	# TITULO.
	if report_name!='cotacao':
		p_titulo = Paragraph('PEDIDO DE COMPRA - %s'%(d_pedido['num_pedido']), PTITULO)
	else:
		p_titulo = Paragraph('PEDIDO DE COTAÇÃO - %s'%(d_pedido['num_pedido']), PTITULO)
	# CONTATO
	tableContato = criarTabelaContato(cr, uid, d_pedido['partner_address_id'], d_pedido['ac'])
	
	# PREVISAO DE ENTREGA, FORMA PAGAMENTO
	if report_name!='cotacao':
		p_prazo = Paragraph('Previsão de Entrega: %s'%(d_pedido['previsao_entrega']), PNORMAL_L)
		p_pagamento = Paragraph('Forma de Pagamento: %s'%(d_pedido['forma_pagamento'].replace('\n','<br/>')), PNORMAL_L)
		msg1 = Paragraph('solicito a compra do(s) seguinte(s) item(ns):', PNORMAL_L)
		msg2 = ''
	else:
		msg1 = Paragraph('solicito a cotação do(s) seguinte(s) item(ns):', PNORMAL_L)
		msg2 = 'Favor informar: <br/>'
		msg2 += '- prazo de entrega <br/>'
		msg2 += '- forma de pagamento <br/>'
		msg2 += '- forma de entrega <br/>'
		msg2 += '- valor total da compra (discriminar impostos) <br/>'
		msg2 = Paragraph(msg2, PNORMAL_L)
	
	# OBSERVACOES
	p_observacoes = Paragraph('Observações: %s' % (d_pedido['observacoes'].replace('\n','<br/>')), PNORMAL_L)
	# TOTAL
	p_total = Paragraph('* Total: %s'%(openerp_utils.reais(d_pedido['total'])), PTOTAL)
	p_total_ = Paragraph('* Total com impostos', PNORMAL_L)
	spacer = Spacer(0,0.5*cm)

	# STORY ############################################################################################
	# Story eh a lista de elementos que compoe o PDF.
	# Elementos criados acima sao adicionados na story.
	story = []
	story.append(p_titulo)
	story.append(spacer)	
	story.append(tableContato)
	story.append(spacer)
	story.append(msg1)
	story.append(spacer)
	
	# tabela de produtos
	story.append(criarTabelaProdutos(produtos_lines))
	
	# TOTAL
	if report_name!='cotacao':
		story.append( Table(data=[[p_total_,p_total]],
			style=tstyleDefault(moreStyles=pdf_utils.tstylePadding(lpad=0,rpad=0,tpad=3*mm,bpad=0))) )

	story.append(spacer)
	
	if report_name!='cotacao':
		#story.append(p_data)
		if d_pedido['previsao_entrega']:
			story.append(p_prazo)
		if d_pedido['forma_pagamento']:
			story.append(p_pagamento)
	else:
		story.append(msg2)
			
	if d_pedido['observacoes']:
		story.append(spacer)
		story.append(p_observacoes)
	
	story.append(spacer)
	story.append(Paragraph('Atenciosamente,', PNORMAL_R))
	story.append(Paragraph(d_pedido['criador'], PNORMAL_R))
	story.append(Paragraph('Setor de compras', PNORMAL_R))

	if report_name!='cotacao' and d_pedido['aprovador']:
		story.append(spacer)	
		story.append(Paragraph('Aprovador:', PNORMAL_R))
		story.append(spacer)
		story.append(Paragraph('_______________________________________', PASSINATURA))
		story.append(Paragraph(d_pedido['aprovador'], PNORMAL_R))
	
	# Funcao createPdf() monta o PDF.
	# No caso do OpenERP eh necessario definir a flag getPdfData como true, para retornar
	# o conteudo do PDF.
	return pdf_utils.createPdf(fileName=pdf_utils.tempFile(), title='pedido_compra.pdf', 
		pageSize=A4, story=story, 
		leftMargin=1.27*cm, rightMargin=1.27*cm, topMargin=3.2*cm, bottomMargin=2*cm,
		getPdfData=True, onPage=drawFalkerHeader, onPageEnd=pdf_utils.drawPageNumber)

# Desenha cabecalho da Falker no PDF.
# Este metodo eh chamado a partir do evento onPage na funcao createPdf().
def drawFalkerHeader(canvas, document, pageSize=A4, 
		imagePath='addons/custom/falker_logo_grande.gif'):
	nome = d_company['nome']
	endereco = d_company['endereco']
	bairro = d_company['bairro']
	cidade = d_company['cidade']
	estado = d_company['estado']
	cep = d_company['cep']
	fone = d_company['fone']
	
	x = float(pageSize[0])/2 + float(pageSize[0])*0.25
	y = pageSize[1]-1*cm
	
	canvas.setFont('LucidaSans-b', 10)
	canvas.drawCentredString(x, y, "%s" % (nome))
	canvas.setFont('LucidaSans', 8)
	y -= 12
	canvas.drawCentredString(x, y, "%s" % (endereco))
	y -= 10
	canvas.drawCentredString(x, y, "%s - %s - %s - CEP: %s" % (bairro,cidade,estado,cep))
	y -= 10
	canvas.drawCentredString(x, y, "Fone/Fax: %s" % (fone))
	y -= 10
	canvas.drawCentredString(x, y, "CNPJ: %s" % ('07.736.144/0001-50'))
	
	canvas.drawImage(imagePath, x=2.5*cm, y=27.5*cm, width=140, height=41)


def criarTabelaContato(cr, uid, partner_address_id, ac):
	d_dest = pooler.get_pool(cr.dbname).get('res.partner.address').browse(cr,uid,[partner_address_id])[0]
	nome_empresa = 'Fornecedor: '+d_dest.partner_id.name
	from time import localtime
	data = localtime()
	data = data[2],data[1],data[0]
	data = '%s/%s/%s'%(data[0],data[1],data[2])
	
	p_dest = Paragraph(nome_empresa, PNEG_L)
	p_data = Paragraph('Data: '+data, PNORMAL_R)
	if ac:
		p_ac = Paragraph('Prezado(a) %s, '%(ac), PNORMAL_L)
	#p_end = Paragraph('Endereço: %s - %s' % (endereco,bairro), PNORMAL_L)
	#p_end2 = Paragraph('CEP: %s - %s - %s' % (cep,cidade,estado), PNORMAL_L)
	
	contents = []
	contents.append(['', ''])
	contents.append([p_dest, p_data])
	contents.append(['', ''])
	if ac:
		contents.append([p_ac, ''])
	#contents.append([p_end])
	#contents.append([p_end2])
	
	styles = []
	styles += tstyleDefault()
	styles += pdf_utils.tstylePadding(lpad=0,rpad=0,tpad=0,bpad=0)
	t_dest = Table(data=contents, style=styles, colWidths=[None,3.3*cm])
	return t_dest

# Cria tabela de itens.
# Cada elemento da lista eh um dicionario contendo os dados de cada linha:
# {item, cod_falker, desc, part_number, qtd_comprar, preco_unit, subtotal, uom}
def criarTabelaProdutos(produtos_lines):
	import dict_utils
	import rutils
	# Cabecalho da tabela (primeira linha)
	tableHeader = []
	tableHeader.append('Item')
	tableHeader.append('Cód.Falker')
	tableHeader.append('Descrição')
	tableHeader.append('Part Number')
	tableHeader.append('Qtd\ncomprar')
	tableHeader.append('Unidade')
	tableHeader.append('Preço\nUnit.')
	tableHeader.append('Subtotal')
	# Cria dicionario de listas para gerenciar colunas da tabela.	
	d_cols = dict_utils.cdl(tableHeader) # Dicionario onde keys sao os nomes das colunas.
	
	for line in produtos_lines:
		# Obtem dados.
		item = str(line['item'])		
		cod_falker = str(line['cod_falker'])
		desc = str(line['desc'])
		part_number = str(line['part_number'])
		qtd_comprar = str(line['qtd_comprar'])
		preco_unit = str(line['preco_unit'])
		subtotal = str(line['subtotal'])
		uom = str(line['uom'])
		# Adiciona dados nas colunas.
		# Caso algum dado seja vazio, paragrafo nao eh criado
		# porque ao criar a tabela eh feito teste para 
		# verificar se coluna esta vazia.
		# Quando todas as linhas da coluna sao vazias ('')
		# a coluna eh removida.
		if item:
			d_cols['Item'].append(Paragraph(item, PTABELA))
		else:
			d_cols['Item'].append('')
		if cod_falker:
			d_cols['Cód.Falker'].append(Paragraph(cod_falker, PTABELA))
		else:
			d_cols['Cód.Falker'].append('')
		d_cols['Descrição'].append(Paragraph(desc, PTABELA))
		if part_number:
			d_cols['Part Number'].append(Paragraph(part_number, PTABELA))
		else:
			d_cols['Part Number'].append('')
		if qtd_comprar:
			d_cols['Qtd\ncomprar'].append(Paragraph(qtd_comprar, PTABELA))
		else:
			d_cols['Qtd\ncomprar'].append('')
		if preco_unit:
			d_cols['Preço\nUnit.'].append(Paragraph(preco_unit, PTABELA))
		else:
			d_cols['Preço\nUnit.'].append('')
		if subtotal:
			d_cols['Subtotal'].append(Paragraph(subtotal, PTABELA))
		else:
			d_cols['Subtotal'].append('')
		if uom:
			d_cols['Unidade'].append(Paragraph(uom, PTABELA))
		else:
			d_cols['Unidade'].append('')

	# Remove colunas vazias.
	# Quando todas as linhas da coluna sao vazias ('')
	# a coluna eh removida.
	aux = []
	for col_name in tableHeader:
		col = d_cols[col_name]
		if not rutils.isFilledList(col, '', strip=True):
			aux.append(col_name)
	tableHeader = aux
	d_cols['header'] = tableHeader
	
	# Define a largura das colunas de tamanho fixo.
	# None = coluna de tamanho variavel.
	colWidths = [None] * len(tableHeader)
	for col_i in range(len(tableHeader)):
		col_name = tableHeader[col_i]
		if col_name == 'Item':
			colWidths[col_i] = 1.2*cm
		if col_name == 'Cód.Falker':
			colWidths[col_i] = 3*cm
		if col_name== 'Part Number':
			colWidths[col_i] = 3.5*cm
		if col_name == 'Qtd\ncomprar':
			colWidths[col_i] = 1.5*cm
		if col_name == 'Preço\nUnit.':
			colWidths[col_i] = 2*cm
		if col_name == 'Subtotal':
			colWidths[col_i] = 2*cm
		if col_name == 'Unidade':
			colWidths[col_i] = 1.5*cm
	
	# Cria uma tabela a partir do dicionario, contendo
	# as colunas definidas em d_cols['header'].
	# d_cols['header'] define a ordem das colunas e quais
	# colunas serao usadas para montar a tabela.
	data = dict_utils.createTableFromDic(d_cols)
	# Estilos da tabela.
	style = tstyleDefault(drawGrid=True, 
		moreStyles=[('FONTNAME',(0,0),(-1,0),'LucidaSans-b'), ('FONTSIZE',(0,0),(-1,0),8)])	
	table = Table(data=data, style=style, colWidths=colWidths)
	return table
	
# Define estilos padrao para a tabela.
# drawGrid: True desenha grid na tabela.
# moreStyles: podem ser adicionados estilos adicionais.
def tstyleDefault(drawGrid=False, moreStyles=[]):
	styles = []
	styles.append( ('FONTNAME',(0,0),(-1,-1),'LucidaSans') )
	styles.append( ('FONTSIZE',(0,0),(-1,-1),10) )
	styles.append( ('VALIGN',(0,0),(-1,-1),'MIDDLE') )
	styles.append( ('ALIGNMENT',(0,0),(-1,-1),'CENTER') )
	if drawGrid:
		styles.append( ('GRID',(0,0),(-1,-1),0.5,'black') )
	for style in moreStyles:
		styles.append(style)
	return styles
	
# Define o estilo de um paragrafo.
# alignment: 0=left, 1=center, 2=right, 4=justify
def pstyle(name, fontName, fontSize=10, textColor='black', 
		spaceBefore=0, spaceAfter=0, leading=None, alignment=1):
	if leading==None:
		leading = fontSize
	return ParagraphStyle(name=name, fontName=fontName, fontSize=fontSize, 
		textColor=textColor, spaceBefore=spaceBefore, spaceAfter=spaceAfter,
		leading=leading, alignment=alignment)

# ESTILOS DE PARAGRAFOS
PNEG = pstyle(name='negrito', fontName='LucidaSans-b', fontSize=10)
PNORMAL = pstyle(name='normal', fontName='LucidaSans', fontSize=10)
PDESC = pstyle(name='desc', fontName='LucidaSans', fontSize=10, alignment=0, leading=12)
PTITULO = pstyle(name='titulo', fontName='LucidaSans-b', fontSize=20, alignment=1)
PNEG_L = pstyle(name='negrito', fontName='LucidaSans-b', fontSize=10, alignment=0, leading=12)
PNORMAL_L = pstyle(name='normal', fontName='LucidaSans', fontSize=10, alignment=0, leading=12)
PNORMAL_R = pstyle(name='normal', fontName='LucidaSans', fontSize=10, alignment=2, leading=12)
PTOTAL = pstyle(name='normal', fontName='LucidaSans-b', fontSize=12, alignment=2, leading=14, spaceBefore=1*mm)
PTABELA = pstyle(name='normal', fontName='LucidaSans', fontSize=8)
PASSINATURA = pstyle(name='normal', fontName='Helvetica', fontSize=8, alignment=2, leading=12)

from report.interface import report_rml
from report.interface import toxml
import pooler

class report_custom(report_rml):

	def create_xml(self, cr, uid, ids, datas, context=None):
		self.cr = cr
		self.uid = uid
		self.ids = ids
		self.datas = datas
		return ''

	def create_rml(self, cr, xml, uid, context=None):
		rml = ''
		return rml
		
	report_name = ''
	
	def create_pdf(self, xml):
		from openerp_utils import field2str, getModel, reais
		cr = self.cr
		uid = self.uid
		ids = self.ids
		data = self.datas
		#print 'ids:',ids
		form = data['form']
		debug = False
		if debug:
			print 'data (pdf):',data
		
		# Pedido de compra ou de cotacao.
		global report_name
		report_name = 'compra'
		if form.has_key('report_name'):
			report_name = form['report_name']
		
		# Obtem dados da purchase order.
		# d_pedido eh dicionario com dados do pedido.
		# {num_pedido, data_pedido, previsao_entrega, produtos}
		d_pedido = openerp_utils.getPurchaseOrderData(cr, uid, ids[0])
		d_pedido['data_pedido'] = openerp_utils.sqldate2brdate(d_pedido['data_pedido'])
		try:
			d_pedido['previsao_entrega'] = openerp_utils.sqldate2brdate(d_pedido['previsao_entrega'])
		except:
			d_pedido['previsao_entrega'] = ''
			
		# Complementa d_pedido com dados do form.
		if report_name!='cotacao':
			d_pedido['forma_pagamento'] = field2str(form['forma_pagamento'])
			d_pedido['aprovador'] = field2str(form['aprovador'])
		d_pedido['ac'] = field2str(form['contato'])
		d_pedido['observacoes'] = field2str(form['observacoes'])		
		d_pedido['criador'] = field2str(form['criador'])
		
		
		produtos_lines = []
		order_lines = d_pedido['order_lines']
		# percorre order_lines da purchase order
		for line in order_lines:
			#item, cod_falker, descricao, part_number, qtd_comprar, preco_unit, subtotal
			line_id = line.id
			#if debug: print 'line_id:',line_id
			# inicializa variaveis
			item = 0
			cod_falker = ''
			desc = ''
			part_number = ''
			qtd_comprar = 0
			preco_unit = 0
			subtotal = 0			
			# obtem dados da order_line
			desc = line.name
			produto_id = line.product_id.id
			#if debug: print 'produto_id:',produto_id
			if produto_id:
				produto_model = getModel(cr, uid, produto_id, 'product.product')
				#if debug: print 'produto_model:',produto_model
				cod_falker = produto_model.default_code
				desc = desc.replace('[%s] '%(cod_falker), '')
			item = openerp_utils.field2str(line.line_number)
			#if debug: print 'item:',item
			part_number = line._field_partner_code(cr, uid, [line_id])
			if len(part_number) > 0:
				part_number = part_number.values()[0]
			qtd_comprar = line.product_qty
			# No pedido de compra preenche valores.
			if report_name != 'cotacao':
				preco_unit = reais(line.price_unit)
				subtotal = reais(line.price_subtotal)
			# No pedido de cotacao nao preenche valores.
			else:
				preco_unit = ''
				subtotal = ''
			uom = line.product_uom.name
			if 'unidade' in uom.lower():
				uom = 'Un.'
			produtos_lines.append( {'item':item, 'cod_falker': cod_falker, 'desc':desc,
				'part_number':part_number, 'qtd_comprar':qtd_comprar,'preco_unit':preco_unit,
				'subtotal':subtotal, 'uom':uom} )
			
		return criarPdf(cr, uid, report_name, d_pedido, produtos_lines)

report_custom('report.purchase.order.pedido_compra', 'purchase.order', '', '')


#criarPdf()
#print 'pdf criado!'
