# coding: utf-8
# Autor: Carlos Klock
from reportlab.lib.units import mm
from reportlab.lib.units import cm

######################################################################################
# Folha A4.
PAGE_A4 = (210*mm,  297*mm)
# Folha A4 - Paisagem/Landscape.
PAGE_A4_L = (297*mm, 210*mm)
# Frame que define as margens da folha A4.
FRAME_A4 = {'id':'first', 'x1':2*cm, 'y1':2*cm, 
	'width':PAGE_A4[0]-4*cm, 'height':PAGE_A4[1]-6*cm}
# Frame que define as margens da folha A4 no formato Paisagem/Landscape.
FRAME_A4_L = {'id':'first', 'x1':2*cm, 'y1':2*cm, 
	'width':PAGE_A4_L[0]-4*cm, 'height':PAGE_A4_L[1]-6*cm}
######################################################################################
	
# Cria frame para definir as margens da pagina.
# Retorna string RML.
# id: identificacao do frame.
# pageSize: tamanho da pagina.
# leftMargin: tamanho da margem esquerda (float).
# rightMargin: tamanho da margem direita (float).
# topMargin: tamanho da margem superior (float).
# bottomMargin: tamanho da margem inferior (float).
def frame_dict(id, pageSize=PAGE_A4, leftMargin=2*cm, rightMargin=2*cm, 
		topMargin=2*cm, bottomMargin=2*cm):
	pageW = pageSize[0]
	pageH = pageSize[1]
	x1 = leftMargin
	y1 = bottomMargin
	width = pageW - leftMargin - rightMargin
	height = pageH - topMargin - bottomMargin
	return {'id':id, 'x1':x1, 'y1':y1, 'width':width, 'height':height}
######################################################################################
	
def frame(id, x1, y1, width, height):
	return '<frame id="%s" x1="%s" y1="%s" width="%s" height="%s"/>' % (id, x1, y1, width, height)
######################################################################################

# Cria uma linha para uma tabela RML.
def tableRow(values):
	tr = '<tr>'
	for i in range(len(values)):
		tr += '<td>$var$valor</td>'
	tr += '</tr>'
	for v in values:
		tr = tr.replace('$var$valor', str(v), 1)
	return tr
######################################################################################

# Cria tabela RML.
# tabHeaders: cabecalhos da tabela: uma lista de listas.
# tabContents: conteudo (lista de listas) para converter em RML.
# style: nome do estilo a ser usado na tabela.
def table(tabHeaders, tabContents, style=''):
	if len(tabHeaders)==0:
		raise UserWarning, "Parametro deve ser uma lista de listas!"
	if len(tabContents)==0:
		raise UserWarning, "Parametro deve ser uma lista de listas!"

	if style:
		rmlTable = '<blockTable style="%s">\n' % (style)
	else:
		rmlTable = '<blockTable>\n'
	
	for row in tabHeaders:
		rmlTable += tableRow(row) + '\n'
	for row in tabContents:
		rmlTable += tableRow(row) + '\n'
	rmlTable += '</blockTable>'
	
	return rmlTable
######################################################################################

# Cria elementos de estilo para definir o 'padding' das células de uma tabela.
def tstylePadding(lpad='1mm', rpad='1mm', tpad='1mm', bpad='1mm'):
	styles = ''
	styles += '<blockLeftPadding length="%(lpad)s"/> \n'
	styles += '<blockRightPadding length="%(rpad)s"/> \n'
	styles += '<blockTopPadding length="%(tpad)s"/> \n'
	styles += '<blockBottomPadding length="%(bpad)s"/> \n'
	styles = styles % {'lpad':lpad, 'rpad':rpad, 'tpad':tpad, 'bpad':bpad}
	return styles
######################################################################################

# Cria elementos de estilo para definir a cor das linhas de uma tabela.
def tstyleRowColor(start=0, stop=0, color1='white', color2='whitesmoke'):
	styles = ''
	flag = True
	for i in range(start, stop+1):
		if flag:
			styles += '<blockBackground colorName="%s" start="0,%s" stop="-1,%s"/>\n' % (color1,i,i)
		else:
			styles += '<blockBackground colorName="%s" start="0,%s" stop="-1,%s"/>\n' % (color2,i,i)
		flag = not flag
	return styles

######################################################################################

# Cria um estilo de paragrafo RML.
# fontName: nome da fonte.
# fontSize: tamanho da fonte.
# fontColor: cor da fonte.
def pstyle(name, fontName='Helvetica', fontSize='10', alignment='justify', 
	textColor='black', backColor='white', spaceBefore='0mm', spaceAfter='2mm', 
	parent=''):
	style = '<paraStyle name="%(name)s" fontName="%(fontName)s" fontSize="%(fontSize)s" '
	style += 'alignment="%(alignment)s" textColor="%(textColor)s" backColor="%(backColor)s" '
	style += 'spaceBefore="%(spaceBefore)s" spaceAfter="%(spaceAfter)s" '
	style += 'parent="%(parent)s" '
	style += '/>\n'
	style = style % {'name':name, 'fontName':fontName, 'fontSize':fontSize, 'alignment':alignment,
		'textColor':textColor, 'backColor':backColor, 'spaceBefore':spaceBefore, 'spaceAfter':spaceAfter,
		'parent':parent}
	return style

######################################################################################

# Retorna string RML com estilos de tabela, inseridos entre as tags <blockTableStyle>
# e </blockTableStyle>.
# id: nome do estilo de tabela.
# styles: estilos a adicionar.
# span: cria estilos blockSpan a partir de uma lista de tuplas (xstart, ystart, xstop, ystop).
# useDefaultStyles: se for True usa fonte Helvetica 10, alinhamento centralizado, e grid na tabela.
def tstyle(id, styles='', span=[], useDefaultStyles=False):
	stylesheet = ''
	if useDefaultStyles:
		stylesheet += '<blockFont name="Helvetica" size="10"/> \n'
		stylesheet += '<blockAlignment value="CENTER"/> \n'
		stylesheet += '<blockValign value="MIDDLE"/> \n' # TOP/MIDDLE/BOTTOM
		stylesheet += '<lineStyle kind="GRID" colorName="#000000" thickness="0.3" start="0,0" stop="-1,-1"/> \n'
	stylesheet += styles	
	for sp in span:
		xstart = sp[0]
		ystart = sp[1]
		xstop = sp[2]
		ystop = sp[3]
		stylesheet += '<blockSpan start="%d,%d" stop="%d,%d"/>' % (xstart, ystart, xstop, ystop)
		return '<blockTableStyle id="%s">\n%s\n</blockTableStyle>\n'% (id, stylesheet)

######################################################################################

#rml = frame(id='teste', pageSize=PAGE_A4, leftMargin=2*cm, rightMargin=2*cm, 
#		topMargin=2*cm, bottomMargin=2*cm)
#print rml
#rml = tstyle(id='Estilo1', 
#	styles='<blockLeftPadding length="5mm" start="0,0" stop="-1,-1"/> \n', 
#	span=[(0,0,-1,1)], useDefaultStyles=True)
#print rml
