# -*- coding: utf-8 -*- 
# Criado por Carlos Klock - 30/out/2008
# Ultima atualizacao: 02-jan-2008

import base64
from reportlab.lib import units
import tempfile
import time
import rml_utils

# Folha A4.
PAGE_A4 = (210*units.mm,  297*units.mm)
# Folha A4 - Paisagem/Landscape.
PAGE_A4_L = (297*units.mm, 210*units.mm)
# Folha Carta.
PAGE_LETTER = (215.9*units.mm, 279.4*units.mm)
# Frame que define as margens da folha A4.
FRAME_A4 = {'id':'first', 'x1':2*units.cm, 'y1':2*units.cm, 
	'width':PAGE_A4[0]-4*units.cm, 'height':PAGE_A4[1]-6*units.cm}
# Frame que define as margens da folha A4 no formato Paisagem/Landscape.
FRAME_A4_L = {'id':'first', 'x1':2*units.cm, 'y1':2*units.cm, 
	'width':PAGE_A4_L[0]-4*units.cm, 'height':PAGE_A4_L[1]-6*units.cm}

def frameRml(id, x1, y1, width, height):
	return '<frame id="%s" x1="%s" y1="%s" width="%s" height="%s"/>' % (id, x1, y1, width, height)

def imageRml(img_name, img_data):
	rml = '<image name="%s">' % (img_name)
	rml += img_data
	rml += '</image>\n'
	return rml

## REPORTS ##################################################################################
class Report:	
	# title: titulo do documento.
	# filename: nome do arquivo rml.
	# story: o conteudo principal do arquivo. Fica entre as tags <story> e </story>
	# pageSize: tamanho da pagina. Padrao eh PAGE_A4.
	# frame: tamanho do quadro de conteudo. Esse quadro eh usado para definir as bordas.
	# pageTemplate: conteudo que fica entre as tags <pageTemplate> e </pageTemplate>.
	# stylesheet: estilos que ficam entre as tags <stylesheet> e </stylesheet>
	# images: lista de caminhos das imagens que serao utilizadas para construir o pdf.
	# pageGraphics: conteudo que fica entre as tags <pageGraphics> e </pageGraphics>
	def createReport(self, title, filename,  story,  pageSize=PAGE_A4,  frame=FRAME_A4,  
		pageTemplate='',  stylesheet='',  images=[],  pageGraphics=''):
		
		report = \
"""<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE document SYSTEM "rml_1_0.dtd">

<document filename="$var$filename">

<template pageSize="$var$pageSize" title="$var$title" author="MAMS">
    <pageTemplate id="main">
        $var$frame
        <pageGraphics>
            $var$pageGraphics
        </pageGraphics>
        $var$pageTemplate
    </pageTemplate>
</template>

<images>
$var$images
</images>

<stylesheet>
$var$stylesheet
</stylesheet>

<!-- The story starts below this comment -->
<story>
$var$story
</story>

</document>"""
		
		report = report.replace('$var$filename', filename)
		report = report.replace('$var$title', title)
		report = report.replace('$var$story', story)
		report = report.replace('$var$pageSize', str(pageSize))
		report = report.replace('$var$frame', frameRml(id=frame['id'], 
			x1=frame['x1'], y1=frame['y1'], width=frame['width'], height=frame['height']))
		report = report.replace('$var$pageTemplate', pageTemplate)
		report = report.replace('$var$stylesheet', stylesheet)
		report = report.replace('$var$pageGraphics', pageGraphics)
		
		imgrml = ''
		for tuple in images:
			img_name = tuple[0]
			img_data = tuple[1]
			imgrml += imageRml(img_name, img_data)
		report = report.replace('$var$images', imgrml)
		
		return report

class EliterjReport(Report):
	def __init__(self, titulo, filename, story, stylesheet, pageSize, frame, 
			rodape='', subtitulo1='', subtitulo2='', drawPageNumber=True, drawTitle=True,
			tituloY=1*units.cm, subtitulo1Y=2*units.cm, subtitulo2Y=3*units.cm):
		self.titulo = titulo
		self.filename = filename
		self.story = story
		self.stylesheet = stylesheet
		self.pageSize = pageSize
		self.frame = frame
		self.rodape = rodape
		self.subtitulo1 = subtitulo1
		self.subtitulo2 = subtitulo2
		
		margemEsquerdaX = frame['x1']
		margemDireitaX = frame['x1'] + frame['width']
		centroX = pageSize[0]/float(2)
		centroY = pageSize[1]/float(2)
		def inverterY(y):
			return pageSize[1] - y
		tituloY = inverterY(tituloY)
		subtitulo1Y = inverterY(subtitulo1Y)
		subtitulo2Y = inverterY(subtitulo2Y)
		imageW = 113.9
		imageH = 142.71
		imageX = '2cm'
		imageY = inverterY(0) - imageH + 0.4*units.cm
		rodapeX = centroX
		pageNumberX = margemDireitaX - 3*units.mm
		
		self.img_elite = 'elite.gif'
		self.img_b64 = file2b64('./addons/eliterj/report/elite.gif')
		#self.img_elite = Image('elite.gif', file2b64('/usr/lib/tinyerp-server/addons/eliterj/report/elite.gif'))
		
		# CABECALHO/RODAPE
		pageGraphics = ''
		if drawTitle:
			pageGraphics += '<setFont name="Helvetica-Bold" size="17"/>'
			pageGraphics += '<drawCentredString x="%(centroX)s" y="%(tituloY)s">%(titulo)s</drawCentredString>'
			pageGraphics = pageGraphics % {'titulo':titulo, 'tituloY':tituloY, 'centroX':centroX}
		
		pageGraphics += """<setFont name="Helvetica" size="12"/>
		<drawCentredString x="%(centroX)s" y="%(subtitulo1Y)s">%(subtitulo1)s</drawCentredString>
		<drawCentredString x="%(centroX)s" y="%(subtitulo2Y)s">%(subtitulo2)s</drawCentredString>
		<image x="%(imageX)s" y="%(imageY)s" file="%(image)s" width="%(imageW)s" height="%(imageH)s"/>

		<setFont name="Helvetica" size="8"/>
		<drawCentredString x="%(rodapeX)s" y="1.5cm">%(rodape)s</drawCentredString>""" % {
			'subtitulo1':subtitulo1, 'subtitulo2':subtitulo2, 'rodape':rodape,
			'centroX':centroX, 'subtitulo1Y':subtitulo1Y, 'subtitulo2Y':subtitulo2Y, 
			'imageX':imageX, 'imageY':imageY, 'image':self.img_elite, 'imageW':imageW, 'imageH':imageH, 
			'rodapeX':rodapeX}
		
		if drawPageNumber:
			pageGraphics += '\n'
			pageGraphics += '<drawCentredString x="%(pageNumberX)s" y="1.5cm"><pageNumber /></drawCentredString>' % {
				'pageNumberX':pageNumberX}
		
		# Cria o relatorio
		self.rml = self.createReport(title=titulo, filename=filename, story=story, pageSize=pageSize, frame=frame, 
			stylesheet=stylesheet, images=[(self.img_elite, self.img_b64)],  pageGraphics=pageGraphics)

	def getRml(self):
		return self.rml


class EtiquetasReport(Report):
	# Cria uma tabela de etiquetas em RML, onde cada etiqueta vai ter o conteudo especificado
	# em rmlContent.
	# rmlContent: lista de tuplas, onde cada tupla = (x, y, conteudo)
	# width: largura da etiqueta.
	# height: altura da etiqueta.
	# ncols: numero de colunas: quantidade horizontal de etiquetas na pagina.
	# nrows: numero de linhas: quantidade vertical de etiquetas na pagina.
	def criarTabela(self, rmlContent, width, height, ncols, nrows):
		# Tamanho das colunas da tabela.
		colWidths = 'colWidths="'
		for i in range(ncols):
			colWidths += width+','
		colWidths = colWidths.strip(',')
		colWidths += '"'
		
		# Tamanho das linhas da tabela.
		rowHeights = 'rowHeights="'
		for i in range(nrows):
			rowHeights += height+','
		rowHeights = rowHeights.strip(',')
		rowHeights += '"'
		
		# Tabela RML.
		table = '<blockTable style="Etiqueta" %(colWidths)s %(rowHeights)s> \n' % {
			'colWidths':colWidths, 'rowHeights':rowHeights}		
		# Cria matriz.
		table_data = matrix(ncols=ncols, nrows=nrows, startVal='')
		# Insere conteudo na matriz.
		for tupla in rmlContent:
			x = tupla[0]
			y = tupla[1]
			content = tupla[2]
			table_data[y][x] = content
		# Converte matriz em RML.
		for row in table_data:
			table += rmlTableRow(row) + '\n'
		table += '</blockTable>\n'
		
		return table

	# Cria RML completo para geracao das etiquetas.
	# dic_etiquetas: num_etiqueta => dados_etiqueta
	#   dados_etiqueta é dicionário com as seguintes chaves:
	#   nome_empresa, a_c, rua, bairro, cep, cidade, uf
	def criarReport(self, dic_etiquetas):
		#nome_empresa='NOME DA EMPRESA', a_c='A/C NOME DO RESPONSÁVEL', rua='Rua', 
		#bairro='Bairro', cep='CEP', cidade='Cidade', uf='UF', etiquetas=[1,2,3,4,5,6]
		rmlContent = []
		
		for num_etiqueta in dic_etiquetas:
			try:
				etiqueta = dic_etiquetas[num_etiqueta]
				nome_empresa = etiqueta['nome_empresa']
				a_c = etiqueta['a_c']
				rua = etiqueta['rua']
				bairro = etiqueta['bairro']
				cep = etiqueta['cep']
				cidade = etiqueta['cidade']
				uf = etiqueta['uf']
			except:
				print 'Dicionário vazio ou inválido!'
				return
			
			texto = ''
			#texto += '<illustration width="30" height="30">'
			#texto += '<image file="elite.gif" x="0" y="0" width="30" height="30"/>'
			#texto += '</illustration>'
			texto += '<para style="NegritoItalico">Destinatário:</para>\n'
			texto += '<spacer length="0.6cm"/>\n'
			texto += '<para style="Normal">%s</para>\n' % (nome_empresa)
			texto += '<spacer length="0.6cm"/>\n'
			texto += '<para style="Normal">A/C %s</para>\n' % (a_c)
			texto += '<spacer length="0.6cm"/>\n'
			texto += '<para style="Normal">%s</para>\n' % (rua)
			texto += '<para style="Normal">%s</para>\n' % (bairro)
			texto += '<para style="Normal">%s</para>\n' % (cep)
			texto += '<para style="Normal">%s - %s</para>\n' % (cidade, uf)
			#texto += '<spacer length="0.7cm"/>\n'
			
			if num_etiqueta==1:
				rmlContent.append((0,0,texto))
			elif num_etiqueta==2:
				rmlContent.append((1,0,texto))
			elif num_etiqueta==3:
				rmlContent.append((0,1,texto))
			elif num_etiqueta==4:
				rmlContent.append((1,1,texto))
			elif num_etiqueta==5:
				rmlContent.append((0,2,texto))
			elif num_etiqueta==6:
				rmlContent.append((1,2,texto))
		## END FOR ##
		
		table = self.criarTabela(rmlContent=rmlContent, width='101.6mm', height='84.67mm', ncols=2, nrows=3)
		
		styleLeft = self.tstyleEtiquetas(background='white', lpad='5mm', rpad='7.5mm', tpad='5mm', bpad='5mm',
			start='0,0', stop='0,-1')
		styleRight = self.tstyleEtiquetas(background='white', lpad='7.5mm', rpad='5mm', tpad='5mm', bpad='5mm',
			start='1,0', stop='1,-1')
		styles = styleLeft + styleRight
		styles = tableStyles(id='Etiqueta', extraStyles=styles, empty=True)
		styles += self.pstyleNormal()
		styles += self.pstyleNormal(name='Negrito', fontName='Helvetica-Bold')
		styles += self.pstyleNormal(name='NegritoItalico', fontName='Helvetica-BoldOblique')
		
		# TODO: ALTERAR O CAMINHO DA IMAGEM
		#image = ('elite.gif', file2b64('/home/carlosklock/python/elite.gif'))
		
		frame = rml_utils.frame_dict(id='main', pageSize=PAGE_LETTER, 
			leftMargin=4*units.mm, rightMargin=4*units.mm, topMargin=1*units.cm, bottomMargin=1*units.cm)
		
		rml = self.createReport(title='Etiquetas', filename='etiqueta.pdf', story=table, 
			stylesheet=styles, pageSize=PAGE_LETTER, frame=frame)
		return rml


	# Define a formatacao/estilo das etiquetas.
	# background: cor de fundo.
	# lpad: tamanho da margem esquerda.
	# rpad: tamanho da margem direita.
	# tpad: tamanho da margem superior.
	# bpad: tamanho da margem inferior.
	# lineColor: cor da linha de borda.
	# lineSize: tamanho da linha de borda.
	def tstyleEtiquetas(self, background='white', 
		lpad='3mm', rpad='3mm', tpad='3mm', bpad='3mm', 
		start='0,0', stop='-1,-1'):
		#lineColor='black', lineSize='1', drawLines=False):
			
		styles = ''
		styles += '<blockAlignment value="CENTER"/> \n'
		styles += '<blockValign value="MIDDLE"/> \n' # TOP/MIDDLE/BOTTOM
		styles += '<blockBackground colorName="%(background)s" start="%(start)s" stop="%(stop)s"/> \n'
		styles += '<blockLeftPadding length="%(lpad)s" start="%(start)s" stop="%(stop)s"/> \n'
		styles += '<blockRightPadding length="%(rpad)s" start="%(start)s" stop="%(stop)s"/> \n'
		styles += '<blockTopPadding length="%(tpad)s" start="%(start)s" stop="%(stop)s"/> \n'
		styles += '<blockBottomPadding length="%(bpad)s" start="%(start)s" stop="%(stop)s"/> \n'
		#styles += '<lineStyle kind="GRID"/> \n'
		styles = styles % {'lpad':lpad, 'rpad':rpad, 'tpad':tpad, 'bpad':bpad,
			'background':background, 'start':start, 'stop':stop}
		
		return styles
	
	def pstyleNormal(self, name="Normal", fontName="Helvetica", fontSize="14", textColor="black"):
		return rml_utils.pstyle(name=name, fontName=fontName, fontSize=fontSize, 
			textColor=textColor, alignment='left')

## FIM DOS REPORTS ##################################################################################

# Retorna uma string codificada em base64 a partir
# do arquivo especificado.
def file2b64(filename):
	f = open(filename)
	img = f.read()
	f.close()
	return base64.b64encode(img)

def b642file(b64str, filename):
	dec = base64.b64decode(b64str)
	f = open(filename, 'w')
	f.write(dec)
	f.close()
	
###############################################

def matrix(ncols, nrows, startVal=None):
	matrix = []
	for y in range(nrows):
		matrix.append([startVal] * ncols)
	return matrix

###############################################

# Cria linha de tabela RML.
def rmlTableRow(values):
	tr = '<tr>'
	for i in range(len(values)):
		tr += '<td>$var$valor</td>'
	tr += '</tr>'
	for v in values:
		tr = tr.replace('$var$valor', str(v), 1)
	return tr

# Cria tabela RML.
# tabHeaders: cabecalhos da tabela: uma lista de listas.
# tabContents: conteudo (lista de listas) para converter em RML.
# style: nome do estilo a ser usado na tabela.
def rmlTable(tabHeaders, tabContents, style=''):
	if len(tabHeaders)==0:
		raise UserWarning, "Parametro deve ser uma lista de listas!"
	if len(tabContents)==0:
		raise UserWarning, "Parametro deve ser uma lista de listas!"

	if style:
		rmlTable = '<blockTable style="%s">\n' % (style)
	else:
		rmlTable = '<blockTable>\n'
	
	for row in tabHeaders:
		rmlTable += rmlTableRow(row) + '\n'
	for row in tabContents:
		rmlTable += rmlTableRow(row) + '\n'
	rmlTable += '</blockTable>'
	
	return rmlTable

# Retorna string RML com estilos padrao de tabela.
# Estes estilos devem ser inseridos entre as tags <blockTableStyle>
# e </blockTableStyle>.
# id: se definido, coloca os estilos entre as tags <blockTableStyle>, 
#    definindo o nome do estilo atraves do nome especificado em id.
# extraStyles: permite adicionar mais estilos.
# span: cria estilos blockSpan a partir de tuplas (xstart, ystart, xstop, ystop)
# empty: True cria blockTableStyle vazio. False cria blockTableStyle com estilos padrao.
def tableStyles(id='', extraStyles='', span=(), empty=False):
	stylesheet = ''
	if not empty:
		stylesheet += '<blockFont name="Helvetica" size="10"/> \n'
		stylesheet += '<blockAlignment value="CENTER"/> \n'
		stylesheet += '<blockValign value="MIDDLE"/> \n' # TOP/MIDDLE/BOTTOM
		stylesheet += '<lineStyle kind="GRID" colorName="#000000" thickness="0.3" start="0,0" stop="-1,-1"/> \n'
	stylesheet += extraStyles
	
	for sp in span:
		xstart = sp[0]
		ystart = sp[1]
		xstop = sp[2]
		ystop = sp[3]
		stylesheet += '<blockSpan start="%d,%d" stop="%d,%d"/>' % (xstart, ystart, xstop, ystop)
		
	if id:
		return '<blockTableStyle id="%s">\n%s</blockTableStyle>\n'% (id, stylesheet)
	else:
		return stylesheet

###############################################
# Faz listas de span a partir de list_span.
#
# Retorna um dicionario, onde:
#   'header' => lista com o texto reproduzido de acordo
#   com a quantidade de colunas que ele ocupa.
#   'span' => lista com tuplas de dois elementos, onde
#   o primeiro elemento eh o indice de inicio do span e
#   o segundo elemento eh o indice de fim do span.
#
# list_span: lista de tuplas, onde:
#   tupla[0] = texto a reproduzir.
#   tupla[1] = numero de colunas.
def span(list_span):
	header = []
	span = []
	k = 0
	for tupla in list_span:
		texto = tupla[0]
		n_cols = tupla[1]
		start = k
		for i in range(n_cols):
			header.append(texto)
			k += 1
		end = k - 1
		span.append((start,end))
	return {'header':header, 'span':span}

###############################################

# Cria indices para uma tabela (lista de listas), onde o nome
# da coluna aponta para o numero da coluna e o nome da linha
# aponta para o numero da linha.
# A ordem das listas de nomes vai definir a ordem das linhas/colunas.
# col_names: lista de nomes das colunas.
# row_names: lista de nomes das linhas.
# startX: valor de inicio da contagem das colunas.
# startY: valor de inicio da contagem das linhas.
def createTableIndex(col_names, row_names, startX, startY):
	num_cols = len(col_names)
	num_rows = len(row_names)
	dict_cols = {}
	dict_rows = {}
	k=0
	for x in range(startX, startX+num_cols):
		name = col_names[k]
		dict_cols[name] = x
		k+=1
	k=0
	for y in range(startY, startY+num_rows):
		name = row_names[k]
		dict_rows[name] = y
		k+=1
	return {'dict_cols':dict_cols, 'dict_rows':dict_rows}

# Retorna a primeira linha da tabela table onde a coluna column tem o valor val.
# table: a tabela (matriz).
# column: a coluna da tabela onde o valor deve ser procurado.
# val: o valor a ser encontrado.
def findRowInTable(table, column, val):
	for row in table:
		foundVal = row[column]
		if foundVal==val:
			return row
			
# Retorna todas as linhas da tabela table onde a coluna column tem o valor val.
def findRowsInTable(table, column, val):
	if type(table)!=type([]):
		raise TypeError, 'table deve ser uma matriz!'
		
	rows = []
	for row in table:
		#print 'row = '+str(row)
		foundVal = row[column]
		if type(foundVal)!=type(val):
			raise TypeError, 'Nao pode comparar tipos diferentes!'
		if foundVal==val:
			rows.append(row)
	return rows

# Retorna valores de uma coluna da tabela.
def getValuesInCol(table, column):
	if type(table)!=type([]):
		raise ValueError('table deve ser lista de listas!')
	if type(column)!=type(0):
		raise ValueError('column deve ser do tipo int!')
	values = []
	for row in table:
		v = row[column]
		values.append(v)
	return values

# Retorna valores nao repetidos de uma coluna da tabela.
def getUniqueValuesInCol(table, column):
	if type(table)!=type([]):
		raise ValueError('table deve ser lista de listas!')
	if type(column)!=type(0):
		raise ValueError('column deve ser do tipo int!')
	values = []
	for row in table:
		v = row[column]
		if not v in values:
			values.append(v)
	return values
	
# Retorna uma nova tabela trocando colunas por linhas.
def cols2rows(table):
	t = []	
	col = table[0]
	ncols = len(table)
	nrows = len(col)
	for y in range(nrows):
		row = []
		for x in range(ncols):
			row.append(table[x][y])
		t.append(row)
	return t

###############################################

def createRowResultado(inscricao, total, provas):
		cols = 2 + len(provas)
		dados = [inscricao, total]
		for p in provas:
			dados.append(p)
		return rmlTableRow(dados)

def createSqlOr(col_name, or_values):
	aux = col_name+'=$ OR '
	selection = ''
	for val in or_values:
		selection += aux.replace('$', "'"+str(val)+"'", 1)
	selection = selection.strip(' OR ')
	return selection
	
def toString(list):
	s = ''
	for x in list:
		s += str(x)
	return s

# Testa se a coluna da tabela esta preenchida com o valor val.
# table: uma lista de listas.
# column: o indice da coluna desejada.
# val: o valor para testar.
# Retorna true se todos os valores da coluna sao iguais a val.
# Retorna false se algum valor for diferente de val.
def isFilled(table, column, val):
	if type(table)!=type([]):
		raise ValueError('table deve ser uma lista de listas!')
	if type(column)!=type(0):
		raise ValueError('column deve ser do tipo int!')
	for row in table:
		foundVal = row[column]
		if foundVal!=val:
			return False
	return True

# Testa se a lista esta preenchida com o valor val.
# myList: lista a ser varrida.
# val: valor que sera procurado.
# strip: define se espacos em branco ao redor dos valores
#  da lista serao removidos antes de testar.
def isFilledList(myList, val, strip=False):
	if type(myList)!=type([]):
		raise ValueError('myList deve ser uma lista!')
	for v in myList:
		if strip and type(v)==type(''):
			v = v.strip()
		if v!=val:
			return False
	return True

def roundFloat1(float_):
	float_ = float(float_)
	return '%(x)0.1f'%{'x':float_}

def roundFloat2(float_):
	float_ = float(float_)
	return '%(x)0.2f'%{'x':float_}

def findFirst(val, lista):
	return lista.index(val)

def findLast(val, lista):
	pos = -1
	lastIndex = len(lista) - 1
	for i in range(lastIndex, -1, -1):
		v = lista[i]
		if v==val:
			pos = i
			break
	if pos==-1:
		raise UserWarning, "Value not found!"
	return pos
	
# Faz split usando pipe '|' para dividir o texto em diferentes campos.
# Retorna uma lista com o conteudo de cada campo, ou apenas o elemento
# do campo desejado.
# text: o texto que vai ser dividido.
# i: o indice do campo desejado. i < 0 retorna todos os campos.
def pipeSplit(text, i=-1):
	x = text.split('|')
	if i>=0:
		return x[i].strip()
	else:
		return stripAll(x)
		
def stripAll(lista):
	aux = []
	for x in lista:
		aux.append(x.strip())
	return aux
	

# Retorna dicionario contendo span de cada materia para criar celulas do header.
# materia_id => span_start, span_end
def materiasHeaderSpan(materias_header, materias, d_materias):	
	d_span = {}
	for id in materias:
		nome_mat = d_materias[id]
		start = findFirst(nome_mat, materias_header)
		end = findLast(nome_mat, materias_header)
		d_span[id] = (start,end)
	return d_span

## A partir de uma string no formato RML, cria um arquivo PDF.
## outputFile: nome do arquivo de saida (opcional).
#def toPdf(rml, outputFile=None):
#	if type(rml)==file:
#		rml.seek(0)
#		rml = rml.read()
#	
#	inputFile = tempfile.NamedTemporaryFile(mode='w')
#	inputFile.write(rml)
#	inputFile.seek(0)
#	
#	if outputFile==None:
#		outputFile = tempfile.NamedTemporaryFile()
#	else:
#		outputFile = open(outputFile, 'w+b')
#	
#	print 'PDF start time:',time.localtime()[3:6]
#	rml2pdf.go(xmlInputName=inputFile.name, outputFileName=outputFile.name)
#	print 'PDF end time:',time.localtime()[3:6]
#	
#	outputFile.seek(0)
#	pdf = outputFile.read()
#	
#	inputFile.close()
#	outputFile.close()
#	return pdf
	
# Escreve no arquivo.
# data: os dados a escrever.
# filename: o nome do arquivo onde os dados serao escritos.
def write(data, filename):
	f = open(filename, 'w+b')
	f.write(data)
	return f


#table = [(1,2,3),(4,5,6),(4,8,9)]
#print findRowsInTable(table, 1, 5)
#print getValuesInCol(table, 2)
#print createSqlOr('prova_id', [4,5,49])
#print toString([1,2,3,4])
#print roundFloat2(0.36633333)
#print findFirst(1, [1,1,1])
#print findLast(1, [1,1,1])
#print pipeSplit('  Ciclo 1    |    Prova 1   ')
#
#d = createTableIndex(row_names=['Portugues','Matematica','Quimica'], 
#	col_names=['P1','P2','P3'], startX=0, startY=0)
#print d['dict_rows']
#print d['dict_cols']
#
#span = span([('teste1', 2), ('abcd', 3)])
#header = span['header']
#print span['span']
#rml = rmlTable([header], [[1,2,3,4,5]])
#print rml
#
#
#dic_etiquetas = {}
#dic_etiquetas[1] = {'nome_empresa':'Empresa X', 'a_c':'Esdrubaldo Ubandoldo',
#	'rua':'Rua das Casas, 1234', 'bairro':'Matagal', 'cep':'99999-000', 'cidade':'Cidade Y',
#	'uf':'UF Z'}
#dic_etiquetas[3] = {'nome_empresa':'Empresa X', 'a_c':'Esdrubaldo Ubandoldo',
#	'rua':'Rua das Casas, 1234', 'bairro':'Matagal', 'cep':'99999-000', 'cidade':'Cidade Y',
#	'uf':'UF Z'}	
#dic_etiquetas[6] = {'nome_empresa':'Falker', 'a_c':'Ângela',
#	'rua':'Rua Washington Luiz', 'bairro':'Cidade Baixa', 'cep':'99999-000', 'cidade':'Porto Alegre',
#	'uf':'RS'}
#e = EtiquetasReport()
#rml = e.criarReport(dic_etiquetas)
#print
#print rml
#print matrix(3, 10, 'X')
#print sqldate2brdate('2008-12-01')
#
#myList = [' a ','a  ','a','a']
#print isFilledList(myList, 'a', strip=True)
