# coding=utf-8
from osv import osv, fields

import time
import netsvc
from mx import DateTime

class sale_order(osv.osv):
	_inherit = "sale.order"

	_columns = {
		'name': fields.char('Order Description', size=128, required=True, select=True),
		'origin': fields.char('Origin', size=64, readonly=True),
	}
	_defaults = {
		'note': lambda *a: '''------------------------------------------------------------
**  Dados complementares sobre equipamentos
------------------------------------------------------------
N.S.:

------------------------------------------------------------
** Informações sobre Frete
------------------------------------------------------------
Valor:
Envio:
Pagamento do frete:

------------------------------------------------------------
** Informações sobre Comissão de Venda
------------------------------------------------------------
Percentual:
Valor:
Parceiro:


------------------------------------------------------------
**  Informações sobre Pagamento
------------------------------------------------------------
Forma de Pagamento:
Prazo:

------------------------------------------------------------
** Informações Complementares
------------------------------------------------------------''',
	}

	#Locations de estoque na Sale Order (na verdade, na Packing gerada pela Sale Order - Sending Goods);
	#No cabeçalho: Location=EXPEDIÇÃO; Final Location=CLIENTES; Dest. Location=CLIENTES;;
	#Na parte central, nas linhas onde têm os produtos colocar: Source Location: PRODUTO ACABADO; Det. Location=EXPEDIÇÃO)	
	def action_ship_create(self, cr, uid, ids, *args):

		def get_location_id(cr, name):
			cr.execute("SELECT id FROM stock_location WHERE name='" + name + "' LIMIT 1")
			res = cr.fetchone()
			if res:
				return res[0]
			else:
				return False

		picking_id=False
		for order in self.browse(cr, uid, ids, context={}):

			location_output = order.shop_id.warehouse_id.lot_output_id.id #Expedição
			location_products = order.shop_id.warehouse_id.lot_destination_id.id #Produto Acabado
			cr.execute("SELECT id FROM stock_location WHERE usage='customer' LIMIT 2")
			res = cr.dictfetchall()
			if len(res)!=1:
				raise UserWarning, 'Sale Order: Location "usage=customer" error!'
			location_customer = res[0]['id'] #Clientes

			picking_id = False
			for line in order.order_line:
				proc_id=False
				date_planned = (DateTime.now() + DateTime.RelativeDateTime(days=line.delay or 0.0)).strftime('%Y-%m-%d')
				if line.state == 'done':
					continue
				if line.product_id and line.product_id.product_tmpl_id.type in ('product', 'consu'):
					location_id = order.shop_id.warehouse_id.lot_stock_id.id
					if not picking_id:
						#loc_dest_id = order.partner_id.property_stock_customer.id
						picking_id = self.pool.get('stock.picking').create(cr, uid, {
							'origin': order.name,
							'type': 'out',
							'state': 'auto',
							'move_type': order.picking_policy,
							'loc_move_id': location_customer,
							'sale_id': order.id,
							'address_id': order.partner_shipping_id.id,
							'note': order.note,
							'invoice_state': (order.order_policy=='picking' and '2binvoiced') or 'none',
							'location_id': location_output,
							'location_dest_id': location_customer,
						})
					
					# Nome do Move mudado para o nome do Sale Order
					move_id = self.pool.get('stock.move').create(cr, uid, {
						'name': order.name,
						'picking_id': picking_id,
						'product_id': line.product_id.id,
						'date_planned': date_planned,
						'product_qty': line.product_uom_qty,
						'product_uom': line.product_uom.id,
						'product_uos_qty': line.product_uos_qty,
						'product_uos': (line.product_uos and line.product_uos.id)\
								or line.product_uom.id,
						'product_packaging' : line.product_packaging.id,
						'address_id' : line.address_allotment_id.id or order.partner_shipping_id.id,
						'location_id': location_products,
						'location_dest_id': location_output,
						'sale_line_id': line.id,
						'tracking_id': False,
						'state': 'waiting',
						'note': line.notes,
					})
					proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
						'name': order.name,
						'origin': order.name,
						'date_planned': date_planned,
						'product_id': line.product_id.id,
						'product_qty': line.product_uom_qty,
						'product_uom': line.product_uom.id,
						'location_id': location_products,
						'procure_method': line.type,
						'move_id': move_id,						
						'property_ids': [(6, 0, [x.id for x in line.property_ids])],
						'from_sale_order': True, # Indica que o Procurement é de Sale Order
					})
					wf_service = netsvc.LocalService("workflow")
					wf_service.trg_validate(uid, 'mrp.procurement', proc_id, 'button_confirm', cr)
					self.pool.get('sale.order.line').write(cr, uid, [line.id], {'procurement_id': proc_id})
				elif line.product_id and line.product_id.product_tmpl_id.type=='service':
					proc_id = self.pool.get('mrp.procurement').create(cr, uid, {
						'name': line.name,
						'origin': order.name,
						'date_planned': date_planned,
						'product_id': line.product_id.id,
						'product_qty': line.product_uom_qty,
						'product_uom': line.product_uom.id,
						'location_id': order.shop_id.warehouse_id.lot_stock_id.id,
						'procure_method': line.type,
						'property_ids': [(6, 0, [x.id for x in line.property_ids])],
					})
					wf_service = netsvc.LocalService("workflow")
					wf_service.trg_validate(uid, 'mrp.procurement', proc_id, 'button_confirm', cr)
					self.pool.get('sale.order.line').write(cr, uid, [line.id], {'procurement_id': proc_id})
				else:
					#
					# No procurement because no product in the sale.order.line.
					#
					pass

			val = {}
			if picking_id:
				wf_service = netsvc.LocalService("workflow")
				wf_service.trg_validate(uid, 'stock.picking', picking_id, 'button_confirm', cr)
				#val = {'picking_ids':[(6,0,[picking_id])]}

			if order.state=='shipping_except':
				val['state'] = 'progress'
				if (order.order_policy == 'manual') and order.invoice_ids:
					val['state'] = 'manual'
			self.write(cr, uid, [order.id], val)
		return True
sale_order()

class sale_order_line(osv.osv):
	_inherit = 'sale.order.line'

	def _delay_date(self, cr, uid, ids, field_name, arg, context):
		res = {}
		for sid in ids:
			res[sid] = False
			line = self.read(cr, uid, [sid], ['delay','order_id'])[0]
			if line:
				sale_order = self.pool.get('sale.order').read(cr, uid, [line['order_id'][0]], ['date_order'])[0]
				tmp = time.mktime(time.strptime(sale_order['date_order'],"%Y-%m-%d")) + line['delay']*24*3600
				tmp = time.gmtime(tmp)
				res[sid] = time.strftime("%Y-%m-%d", tmp)
		return res

	_columns = {
		'delay_date': fields.function(_delay_date, method=True, type='date', string='Delivery Date'),
	}

sale_order_line()
