# coding=utf-8
from osv import osv, fields

# Fix um bug do Tiny aumentado o tamanho do campo para 128 chrs.
class stock_move(osv.osv):
	_inherit = "stock.move"
	_columns = {
		'name': fields.char('Name', size=128, required=True, select=True),
	}
	
	# Corrige o BUG do DRAFT MOVE criado pelo MOVE LOT.
	def copy(self, cr, uid, id, default=None, context=None):
		if not context:
			context={}
		if not default:
			default = {}
		if 'state' not in default:
			if 'state' in self._defaults:
				default['state'] = self._defaults['state'](self, cr, uid, context)
		data = self.read(cr, uid, [id], context=context)[0]
		fields = self.fields_get(cr, uid)
		for f in fields:
			ftype = fields[f]['type']

 			if self._log_access and f in ('create_date', 'create_uid', 'write_date', 'write_uid'):
 				del data[f]

			if f in default:
				data[f] = default[f]
			elif ftype == 'function':
				del data[f]
			elif ftype == 'many2one':
				try:
					data[f] = data[f] and data[f][0]
				except:
					pass
			elif ftype in ('one2many', 'one2one'):
				res = []
				rel = self.pool.get(fields[f]['relation'])
				for rel_id in data[f]:
					#res.append((4, rel.copy(cr, uid, rel_id, context=context)))
					pass
				data[f] = res
			elif ftype == 'many2many':
				data[f] = [(6, 0, data[f])]
		del data['id']
		for v in self._inherits:
			del data[self._inherits[v]]
		return self.create(cr, uid, data)
			
stock_move()

# Cria novo campo em Warehouse para ser usado como Produção (mesa, linha de montagem) em Production.
class stock_warehouse(osv.osv):
	_inherit = "stock.warehouse"
	_columns = {
		'lot_production_id': fields.many2one('stock.location', 'Produção', required=True ),
		'lot_origin_id': fields.many2one('stock.location', 'Origem', required=True ),
		'lot_destination_id': fields.many2one('stock.location', 'Destino', required=True ),
	}
stock_warehouse()

# Cria uma warehouse associada a cada location (opcional, por causa dos limbos)
class stock_location(osv.osv):
	_inherit = "stock.location"
	_columns = {
		'warehouse_id': fields.many2one('stock.warehouse', 'Warehouse', select=True),
	}
stock_location()

class stock_picking(osv.osv):
	_inherit = "stock.picking"
	_columns = {
		'origin': fields.char('Origin', size=64, readonly=True),
		
		# Aponta para uma Purchase. Não é obrigatório. Serve para a rastreabilidade do desfazer.
		'purchase_id': fields.many2one('purchase.order', 'Purchase', readonly=True),
	}


	def action_confirm(self, cr, uid, ids, *args):
		self.write(cr, uid, ids, {'state': 'confirmed'})
		todo = []
		for picking in self.browse(cr, uid, ids):
			number = self.pool.get('ir.sequence').get(cr, uid, 'stock.picking.%s' % picking.type)
			self.write(cr, uid, [picking.id], {'name': number})
			for r in picking.move_lines:
				if r.state=='draft':
					todo.append(r.id)
				if picking.type == 'internal' and not picking.origin:
					r.write(cr,uid,[r.id],{'name':number})
				
		if len(todo):
			self.pool.get('stock.move').action_confirm(cr,uid, todo)
		return True

stock_picking()
