# coding=utf-8

import wizard
import netsvc
import time
import pooler

from osv import osv

_form_bom = '<?xml version="1.0"?> \
	<form string="Revisar Produto"> \
		<separator string="Árvore de BoM:                                                                                                                                       " colspan="4"/> \
		<field name="bom" colspan="4" nolabel="1"/> <newline /> \
		<separator string="Relação dos Produtos alterados nesta revisão (ex: 3,54,13):" colspan="4"/> \
		<field name="review" colspan="4" nolabel="1"/> \
	</form>'
_fields_bom = {
	'bom': {'string':'', 'type':'text', 'default': 'default', 'readonly':False},
	'review': {'string':'', 'type':'char', 'size':128, 'default': '', 'required': True},
}
_form_verify = '<?xml version="1.0"?>\
	<form string="Revisar Produto">\
		<separator string="Verifique os produtos a serem revisados:                                                                                         " colspan="4"/>\
		<field name="verify" colspan="4" nolabel="1"/><newline/>\
	</form>'
_fields_verify = {
	'verify': {'string':'', 'type':'text', 'default': 'default', 'readonly':True},
}
_form_prompt = '<?xml version="1.0"?> \
	<form string="Revisar Produto"> \
		<separator string="Versão atual do produto:                                                                                                         " colspan="4"/> \
		<field name="actual_name" colspan="4"/> \
		<field name="actual_code"/> \
		<field name="actual_qty"/> \
		<separator string="Nova revisão do produto:" colspan="4"/> \
		<field name="new_name" colspan="4"/> \
		<field name="new_code"/> \
		<field name="new_qty"/> \
	</form>'
_fields_prompt = {
	'actual_name': {'string':'Nome', 'type':'char', 'size':128, 'readonly':True},
	'actual_code': {'string':'Código', 'type':'char', 'size':128, 'readonly':True},
	'actual_qty': {'string':'Quantidade', 'type':'float', 'readonly':True},
	'new_name': {'string':'Nome', 'type':'char', 'size':128, 'required':True},
	'new_code': {'string':'Código', 'type':'char', 'size':128, 'required':True},
	'new_qty': {'string':'Quantidade', 'type':'float', 'required':True},
}
_form_done = '<?xml version="1.0"?><form string="Revisar Produto"><label string="Concluído!"/></form>'

def _build_bom_tree (main_bom):

	ref_tree = {}
	ref_tree_index = []
	def _get_childs (bom,parent=-1,level=0):
		ref_id = len(ref_tree_index)
		ref_tree_index.append(bom.id)
		
		ref_tree[ref_id] = {
			'name': bom.product_id.product_tmpl_id.name,
			'product_id': bom.product_id.id,
			'code': bom.product_id.default_code,
			'bom_id': bom.id,
			'qty': bom.product_qty,
			'parent': parent,
			'childs': [],				
			'updated': False,
			'level': level,
		}
		for child in bom.bom_lines:
			ref_tree[ref_id]['childs'].append(_get_childs(child,ref_id,level+1))
			
		return ref_id
	
	_get_childs(main_bom)
	
	return [ref_tree,ref_tree_index]

def _view_bom_tree (self, cr, uid, data, context):
	main_bom = pooler.get_pool(cr.dbname).get('mrp.bom').browse(cr, uid, data['ids'], [])[0]

	res = _build_bom_tree(main_bom)

	data['ref_tree'] = res[0]
	data['ref_tree_index'] = res[1]
	
	data['virtual_index'] = []
	_fields_bom['bom']['default'] =  ''
	for i in range(len(data['ref_tree'])):
		offset = ''
		for j in range(data['ref_tree'][i]['level']):
#			offset += '    -       '# alterado por Greff, 24/09/08
			offset += '    ;       '
			
		if len(data['ref_tree'][i]['childs']) > 0:
			k = len(data['virtual_index'])
			data['virtual_index'].append(i)
		else:
			k = 0

		if k > 0: # Pai não-primeiro
#			_fields_bom['bom']['default'] +=  '%03i: %s%s (qty: %.2f)\n' % (k,offset,data['ref_tree'][i]['name'],data['ref_tree'][i]['qty'])# alterado por Greff, 24/09/08
			_fields_bom['bom']['default'] +=  '%03i: %s%.2f; %s\n' % (k,offset,data['ref_tree'][i]['qty'],data['ref_tree'][i]['name'])
		elif i > 0: # Filho
#			_fields_bom['bom']['default'] +=  '        %s%s (qty: %.2f)\n' % (offset,data['ref_tree'][i]['name'],data['ref_tree'][i]['qty'])# alterado por Greff, 24/09/08
			_fields_bom['bom']['default'] +=  '        %s%.2f; %s\n' % (offset,data['ref_tree'][i]['qty'],data['ref_tree'][i]['name'])
		else:  # Pai primeiro
#			_fields_bom['bom']['default'] +=  '# %s (qty: %.2f)\n' % (data['ref_tree'][i]['name'],data['ref_tree'][i]['qty'])# alterado por Greff, 24/09/08
			_fields_bom['bom']['default'] +=  '# %.2f; %s\n' % (data['ref_tree'][i]['qty'],data['ref_tree'][i]['name'])
		i += 1

	return {}


def _read_string (review):
	review = review.split(',')
	tmp = []
	for rev in review:
		wrong_chars = rev.strip('1234567890') 
		rev = rev.strip(wrong_chars)
		if len(rev)>0:
			rev = int(rev)
			if rev>=0:
				tmp.append(rev)
	return tmp

def _verify (self, cr, uid, data, context):	
	virtual_review = _read_string(data['form']['review'])
	data['review'] = []
	value = ''
	for v in virtual_review:
		if v < len(data['virtual_index']) and v >= 0:
			ref = data['virtual_index'][v]
			data['review'].append(ref)
			if ref < len(data['ref_tree']) and ref >= 0:
				value +=  '%03i: %s (qty: %.2f)\n' % (v,data['ref_tree'][ref]['name'],data['ref_tree'][ref]['qty'])

	return {'verify': value}



def _duplicate (self, cr, uid, data, ref):
	if not data['ref_tree'][ref]['updated']:	
		data['ref_tree'][ref]['updated'] = True
		
		bom = pooler.get_pool(cr.dbname).get('mrp.bom').browse(cr, uid, [data['ref_tree'][ref]['bom_id']], [])[0]
		template = bom.product_id.product_tmpl_id
		product = bom.product_id
		
		values = {
			'name': 'REV/' + template.name,			
			'state': 'draft',
			'uom_id':template.uom_id.id,
			'uos_id':template.uos_id.id,
			'categ_id':template.categ_id.id,
			'uom_po_id':template.uom_po_id.id,
			'product_manager':template.product_manager.id,
			'tracking':template.tracking,
			'warranty':template.warranty,
			'supply_method':template.supply_method,
			'list_price':template.list_price,
			'weight':template.weight,
			'standard_price':template.standard_price,
			'description_purchase':template.description_purchase,
			'uos_coeff':template.uos_coeff,
			'type':template.type,
			'description':template.description,
			'volume':template.volume,
			'procure_method':template.procure_method,
			'cost_method':template.cost_method,
			'rental':template.rental,
			'sale_ok':template.sale_ok,
			'sale_delay':template.sale_delay,
			'description_sale':template.description_sale,
			'purchase_ok':template.purchase_ok,
			'mes_type':template.mes_type,
			'produce_delay':template.produce_delay,
			'weight_net':template.weight_net,
			'taxes_id': [(6, 0, [x.id for x in template.taxes_id])],
			'supplier_taxes_id': [(6, 0, [x.id for x in template.supplier_taxes_id])],
		}
		new_template = pooler.get_pool(cr.dbname).get('product.template').create(cr, uid, values)
		data['ref_tree'][ref]['template_id'] = new_template
		
		values = {
			'default_code': 'REV/' + product.default_code,
			'product_tmpl_id': new_template,
			'ean13':product.ean13,
			'active':product.active,
			'variants':product.variants,
			'price_extra':product.price_extra,
			'price_margin':product.price_margin,
		}
		new_product = pooler.get_pool(cr.dbname).get('product.product').create(cr, uid, values)
		data['ref_tree'][ref]['product_id'] = new_product
		
		for sellers in product.seller_ids:
			values = {
				'product_id':new_template,
				'delay':sellers.delay,
				'qty':sellers.qty,
				'name':sellers.name.id,
				'sequence':sellers.sequence,
				'product_code':sellers.product_code,
				'product_name':sellers.product_name,
			}
			new_supplierinfo = pooler.get_pool(cr.dbname).get('product.supplierinfo').create(cr, uid, values)
			
			for pricelist in sellers.pricelist_ids:
				values = {
					'suppinfo_id':new_supplierinfo,
					'name':pricelist.name,
					'min_quantity':pricelist.min_quantity,
					'price':pricelist.price,
				}
				partnerinfo = pooler.get_pool(cr.dbname).get('pricelist.partnerinfo').create(cr, uid, values)

		for pack in product.packaging:			
			values = {
				'product_id':new_product,
				'ul': pack.ul.id,
				'name':pack.name,
				'qty':pack.qty,
				'ul_qty':pack.ul_qty,
				'rows':pack.rows,
				'ean':pack.ean,
				'code':pack.code,
				'weight':pack.weight,
				'weight_ul':pack.weight_ul,
				'height':pack.height,
				'width':pack.width,
				'length':pack.length,
			}
			supplierinfo = pooler.get_pool(cr.dbname).get('product.packaging').create(cr, uid, values)

		# Cria BoM pra ligar nos filhos
		childs = []
		for child in data['ref_tree'][ref]['childs']:
			childs.append(data['ref_tree'][child]['bom_id'])
			
		values = {
			'name': 'REV/' + template.name,
			'product_id': new_product,
			'code': bom.code,
			'active': bom.active,
			'type': bom.type,
			'date_start': bom.date_start,
			'date_stop': bom.date_stop,
			'sequence': bom.sequence,
			'position': bom.position,
			'product_qty': bom.product_qty,
			'product_uom': bom.product_uom.id,
			'product_rounding': bom.product_rounding,
			'product_efficiency': bom.product_efficiency,
			'revision_type': bom.revision_type,
			'bom_lines': [(6,0,childs)],
		}
		new_bom = pooler.get_pool(cr.dbname).get('mrp.bom').create(cr, uid, values)
		data['ref_tree'][ref]['bom_id'] = new_bom
		
		# Se o pai já foi atualizado, atualizar link entre pai e novo filho.		
		parent = data['ref_tree'][ref]['parent']
		if parent>=0 and data['ref_tree'][parent]['updated']:
			cr.execute('UPDATE mrp_bom_rel SET child_id=%i WHERE parent_id=%i AND child_id=%i' % (new_bom,data['ref_tree'][parent]['bom_id'],bom.id))
			cr.commit()
		
		data['ref_tree'][ref]['updated'] = True
		data['update'].append(ref)
		
def _update (self, cr, uid, data, ref):
	if not data['ref_tree'][ref]['updated']:	
		data['ref_tree'][ref]['updated'] = True
		
		bom = pooler.get_pool(cr.dbname).get('mrp.bom').browse(cr, uid, [data['ref_tree'][ref]['bom_id']], [])[0]
		template = bom.product_id.product_tmpl_id
		product = bom.product_id
		
		values = {'name': data['form']['new_name']}
		pooler.get_pool(cr.dbname).get('product.template').write(cr, uid, [data['ref_tree'][ref]['template_id']], values)
		
		values = {'default_code': data['form']['new_code']}
		pooler.get_pool(cr.dbname).get('product.product').write(cr, uid, [data['ref_tree'][ref]['product_id']], values)
		
		values = {'name':data['form']['new_name'], 'product_qty': data['form']['new_qty']}
		pooler.get_pool(cr.dbname).get('mrp.bom').write(cr, uid, [data['ref_tree'][ref]['bom_id']], values)

def _confirm (self, cr, uid, data, context):
	
	# Cria toda nova árvore
	if  not data.has_key('prompt'):
		data['update'] = []
		for ref in data['review']:
			
			def _create_node_tree (ref):
				if not data['ref_tree'][ref]['updated']:
					_duplicate (self, cr, uid, data, ref)
					if data['ref_tree'][ref]['parent'] >= 0:
						_create_node_tree (data['ref_tree'][ref]['parent'])
						
			_create_node_tree (ref)
			
		for ref in data['ref_tree']:
			data['ref_tree'][ref]['updated'] = False
	
	# Atualiza dados já criado
	else:
		ref = data['prompt']
		_update (self, cr, uid, data, ref)
		
	if len(data['update']) > 0:
		data['prompt'] = data['update'].pop()
		ref = data['prompt']
		
		# Verifica se o Pai precisa ser revisado
		parent = data['ref_tree'][ref]['parent']
		if parent >= 0:		
			if not data['ref_tree'][parent]['updated']:
				data['review'].append(data['ref_tree'][ref]['parent'])
		
		state = 'prompt'
	else:
		state = 'done'
	return state


def _prompt (self, cr, uid, data, context):
	ref = data['prompt']		
	name = data['ref_tree'][ref]['name']
	qty = data['ref_tree'][ref]['qty']
	code = data['ref_tree'][ref]['code']
	return {'actual_name':name,'actual_code':code,'actual_qty':qty,'new_name':name,'new_code':code,'new_qty':qty,}
	

def _action_open_tab (self, cr, uid, data, context):
	if data['ref_tree'][0]['updated']:
		obj = pooler.get_pool(cr.dbname).get('mrp.bom')
		ids = data['ref_tree'][0]['bom_id']
		ids = obj.search(cr, uid, [('id', 'in', [data['ref_tree'][0]['bom_id']] )] )
		cr.execute('select id from ir_ui_view where model=%s and field_parent=%s and type=%s', ('mrp.bom', 'bom_lines', 'tree'))
		view_id = cr.fetchone()[0]
		return {
			'domain': "[('id','in',["+','.join(map(str,ids))+"])]",
			'name': 'BoM Revisada',
			'view_type': 'tree',
			'res_model': 'mrp.bom',
			'field_parent': 'bom_lines',
			'view_id': (view_id,'View'),
			'type': 'ir.actions.act_window'
		}
	else:
		return {}

class wiz_journal(wizard.interface):
	states = {
		'init': {
			'actions': [_view_bom_tree],
			'result': {'type': 'form', 'arch':_form_bom, 'fields':_fields_bom, 'state':[('end','Cancelar'),('verify','Verificar Escolha')]}
		},
		'verify': {
			'actions': [_verify],
			'result': {'type': 'form', 'arch':_form_verify, 'fields':_fields_verify, 'state':[('end','Cancelar'),('init','Voltar'),('confirm','Confirmar Revisão')]}
		},
		'confirm': {
			'actions': [],
			'result': {'type':'choice', 'next_state': _confirm}
		},		
		'prompt': {
			'actions': [_prompt],
			'result': {'type': 'form', 'arch':_form_prompt, 'fields':_fields_prompt, 'state':[('confirm','Avançar')]}
		},
		'done': {
			'actions': [],
			'result': {'type': 'form', 'arch':_form_done, 'fields':{}, 'state':[('open','Fechar')]}
		},
		'open': {
			'actions': [],
			'result': {'type': 'action', 'action': _action_open_tab, 'state':'end'}
		},
	}
wiz_journal('mrp.bom.review')
